'use strict';

var dom = /*#__PURE__*/Object.freeze({
    __proto__: null,
    get add () { return add; },
    get get () { return get; },
    get getId () { return getId; },
    get getNode () { return getNode; },
    get getValue () { return getValue; },
    get has () { return has$1; },
    get hashText () { return hashText; },
    get iframe () { return iframe; },
    get iframeContent () { return iframeContent; },
    get lookup () { return lookup; },
    get parse () { return parse; },
    get removeIFrame () { return removeIFrame; },
    get sameorigin () { return sameorigin; },
    get start () { return start$j; },
    get stop () { return stop$h; },
    get update () { return update; },
    get updates () { return updates$2; }
});
var upload$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    get queue () { return queue; },
    get start () { return start$g; },
    get stop () { return stop$e; },
    get track () { return track$1; }
});
var upgrade$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    get data () { return data$5; },
    get start () { return start$b; },
    get stop () { return stop$b; },
    get upgrade () { return upgrade; }
});
var limit = /*#__PURE__*/Object.freeze({
    __proto__: null,
    get check () { return check$2; },
    get compute () { return compute$3; },
    get data () { return data$4; },
    get start () { return start$a; },
    get stop () { return stop$a; },
    get trigger () { return trigger; }
});
var dimension = /*#__PURE__*/Object.freeze({
    __proto__: null,
    get compute () { return compute$2; },
    get data () { return data$3; },
    get log () { return log; },
    get reset () { return reset$3; },
    get start () { return start$9; },
    get stop () { return stop$9; },
    get updates () { return updates; }
});
var metadata$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    get callback () { return callback; },
    get callbacks () { return callbacks; },
    get clear () { return clear; },
    get consent () { return consent; },
    get data () { return data$2; },
    get electron () { return electron; },
    get id () { return id; },
    get metadata () { return metadata; },
    get save () { return save; },
    get shortid () { return shortid; },
    get start () { return start$8; },
    get stop () { return stop$8; }
});
var envelope$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    get data () { return data$1; },
    get envelope () { return envelope; },
    get start () { return start$7; },
    get stop () { return stop$7; }
});
var clarity = /*#__PURE__*/Object.freeze({
    __proto__: null,
    get consent () { return consent; },
    get event () { return event; },
    get hashText () { return hashText; },
    get identify () { return identify; },
    get metadata () { return metadata; },
    get pause () { return pause; },
    get resume () { return resume; },
    get set () { return set; },
    get signal () { return signal; },
    get start () { return start; },
    get stop () { return stop; },
    get upgrade () { return upgrade; },
    get version () { return version$1; }
});

const w = window;
const c = "clarity" /* Constant.Clarity */;
function setup() {
    // Start queuing up calls while Clarity is inactive and we are in a browser enviornment    
    if (typeof w !== "undefined") {
        w[c] = function () {
            (w[c].q = w[c].q || []).push(arguments);
            // if the start function was called, don't queue it and instead process the queue
            arguments[0] === "start" && w[c].q.unshift(w[c].q.pop()) && process$8();
        };
    }
}
function process$8() {
    if (typeof w !== "undefined") {
        // Do not execute or reset global "clarity" variable if a version of Clarity is already running on the page
        if (w[c] && w[c].v) {
            return console.warn("Error CL001: Multiple Clarity tags detected.");
        }
        // Expose clarity in a browser environment
        // To be efficient about queuing up operations while Clarity is wiring up, we expose clarity.*(args) => clarity(*, args);
        // This allows us to reprocess any calls that we missed once Clarity is available on the page
        // Once Clarity script bundle is loaded on the page, we also initialize a "v" property that holds current version
        // We use the presence or absence of "v" to determine if we are attempting to run a duplicate instance
        let queue = w[c] ? (w[c].q || []) : [];
        w[c] = function (method, ...args) { return clarity[method](...args); };
        w[c].v = version$1;
        while (queue.length > 0) {
            w[c](...queue.shift());
        }
    }
}

let config$2 = {
    projectId: null,
    delay: 1 * 1000 /* Time.Second */,
    lean: false,
    lite: false,
    track: true,
    content: true,
    drop: [],
    mask: [],
    unmask: [],
    regions: [],
    cookies: [],
    fraud: true,
    checksum: [],
    report: null,
    upload: null,
    fallback: null,
    upgrade: null,
    action: null,
    dob: null,
    delayDom: false,
    throttleDom: true,
    conversions: false,
    includeSubdomains: true,
};

function api(method) {
    // Zone.js, a popular package for Angular, overrides native browser APIs which can lead to inconsistent state for single page applications.
    // Example issue: https://github.com/angular/angular/issues/31712
    // As a work around, we ensuring Clarity access APIs outside of Zone (and use native implementation instead)
    return window["Zone" /* Constant.Zone */] && "__symbol__" /* Constant.Symbol */ in window["Zone" /* Constant.Zone */] ? window["Zone" /* Constant.Zone */]["__symbol__" /* Constant.Symbol */](method) : method;
}

let startTime = 0;
function start$I() {
    startTime = performance.now() + performance.timeOrigin;
}
// event.timestamp is number of milliseconds elapsed since the document was loaded 
// since iframes can be loaded later the event timestamp is not the same as performance.now()
// converting everything to absolute time by adding timeorigin of the event view
// to synchronize times before calculating the difference with start time
function time(event = null) {
    let ts = event && event.timeStamp > 0 ? event.timeStamp : performance.now();
    let origin = event && event.view ? event.view.performance.timeOrigin : performance.timeOrigin;
    return Math.max(Math.round(ts + origin - startTime), 0);
}
function stop$F() {
    startTime = 0;
}

let version$1 = "0.8.9-beta";

// tslint:disable: no-bitwise
function hash (input, precision = null) {
    // Code inspired from C# GetHashCode: https://github.com/Microsoft/referencesource/blob/master/mscorlib/system/string.cs
    let hash = 0;
    let hashOne = 5381;
    let hashTwo = hashOne;
    for (let i = 0; i < input.length; i += 2) {
        let charOne = input.charCodeAt(i);
        hashOne = ((hashOne << 5) + hashOne) ^ charOne;
        if (i + 1 < input.length) {
            let charTwo = input.charCodeAt(i + 1);
            hashTwo = ((hashTwo << 5) + hashTwo) ^ charTwo;
        }
    }
    // Replace the magic number from C# implementation (1566083941) with a smaller prime number (11579)
    // This ensures we don't hit integer overflow and prevent collisions
    hash = Math.abs(hashOne + (hashTwo * 11579));
    return (precision ? hash % Math.pow(2, precision) : hash).toString(36);
}

const catchallRegex = /\S/gi;
const maxUrlLength = 255;
let unicodeRegex = true;
let digitRegex = null;
let letterRegex = null;
let currencyRegex = null;
function text$1(value, hint, privacy, mangle = false, type) {
    if (value) {
        if (hint == "input" && (type === "checkbox" || type === "radio")) {
            return value;
        }
        switch (privacy) {
            case 0 /* Privacy.None */:
                return value;
            case 1 /* Privacy.Sensitive */:
                switch (hint) {
                    case "*T" /* Layout.Constant.TextTag */:
                    case "value":
                    case "placeholder":
                    case "click":
                        return redact$1(value);
                    case "input":
                    case "change":
                        return mangleToken(value);
                }
                return value;
            case 2 /* Privacy.Text */:
            case 3 /* Privacy.TextImage */:
                switch (hint) {
                    case "*T" /* Layout.Constant.TextTag */:
                    case "data-" /* Layout.Constant.DataAttribute */:
                        return mangle ? mangleText(value) : mask(value);
                    case "src":
                    case "srcset":
                    case "title":
                    case "alt":
                        return privacy === 3 /* Privacy.TextImage */ ? "" /* Data.Constant.Empty */ : value;
                    case "value":
                    case "click":
                    case "input":
                    case "change":
                        return mangleToken(value);
                    case "placeholder":
                        return mask(value);
                }
                break;
            case 4 /* Privacy.Exclude */:
                switch (hint) {
                    case "*T" /* Layout.Constant.TextTag */:
                    case "data-" /* Layout.Constant.DataAttribute */:
                        return mangle ? mangleText(value) : mask(value);
                    case "value":
                    case "input":
                    case "click":
                    case "change":
                        return Array(5 /* Data.Setting.WordLength */).join("\u2022" /* Data.Constant.Mask */);
                    case "checksum":
                        return "" /* Data.Constant.Empty */;
                }
                break;
            case 5 /* Privacy.Snapshot */:
                switch (hint) {
                    case "*T" /* Layout.Constant.TextTag */:
                    case "data-" /* Layout.Constant.DataAttribute */:
                        return scrub(value, "\u25AA" /* Data.Constant.Letter */, "\u25AB" /* Data.Constant.Digit */);
                    case "value":
                    case "input":
                    case "click":
                    case "change":
                        return Array(5 /* Data.Setting.WordLength */).join("\u2022" /* Data.Constant.Mask */);
                    case "checksum":
                    case "src":
                    case "srcset":
                    case "alt":
                    case "title":
                        return "" /* Data.Constant.Empty */;
                }
                break;
        }
    }
    return value;
}
function url$1(input, electron = false, truncate = false) {
    let result = input;
    // Replace the URL for Electron apps so we don't send back file:/// URL
    if (electron) {
        result = `${"https://" /* Data.Constant.HTTPS */}${"Electron" /* Data.Constant.Electron */}`;
    }
    else {
        let drop = config$2.drop;
        if (drop && drop.length > 0 && input && input.indexOf("?") > 0) {
            let [path, query] = input.split("?");
            let swap = "*na*" /* Data.Constant.Dropped */;
            result = path + "?" + query.split("&").map(p => drop.some(x => p.indexOf(`${x}=`) === 0) ? `${p.split("=")[0]}=${swap}` : p).join("&");
        }
    }
    if (truncate) {
        result = result.substring(0, maxUrlLength);
    }
    return result;
}
function mangleText(value) {
    let trimmed = value.trim();
    if (trimmed.length > 0) {
        let first = trimmed[0];
        let index = value.indexOf(first);
        let prefix = value.substr(0, index);
        let suffix = value.substr(index + trimmed.length);
        return `${prefix}${trimmed.length.toString(36)}${suffix}`;
    }
    return value;
}
function mask(value) {
    return value.replace(catchallRegex, "\u2022" /* Data.Constant.Mask */);
}
function scrub(value, letter, digit) {
    regex(); // Initialize regular expressions
    return value ? value.replace(letterRegex, letter).replace(digitRegex, digit) : value;
}
function mangleToken(value) {
    let length = ((Math.floor(value.length / 5 /* Data.Setting.WordLength */) + 1) * 5 /* Data.Setting.WordLength */);
    let output = "" /* Layout.Constant.Empty */;
    for (let i = 0; i < length; i++) {
        output += i > 0 && i % 5 /* Data.Setting.WordLength */ === 0 ? " " /* Data.Constant.Space */ : "\u2022" /* Data.Constant.Mask */;
    }
    return output;
}
function regex() {
    // Initialize unicode regex, if supported by the browser
    // Reference: https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/Regular_Expressions/Unicode_Property_Escapes
    if (unicodeRegex && digitRegex === null) {
        try {
            digitRegex = new RegExp("\\p{N}", "gu");
            letterRegex = new RegExp("\\p{L}", "gu");
            currencyRegex = new RegExp("\\p{Sc}", "gu");
        }
        catch (_a) {
            unicodeRegex = false;
        }
    }
}
function redact$1(value) {
    let spaceIndex = -1;
    let gap = 0;
    let hasDigit = false;
    let hasEmail = false;
    let hasWhitespace = false;
    let array = null;
    regex(); // Initialize regular expressions
    for (let i = 0; i < value.length; i++) {
        let c = value.charCodeAt(i);
        hasDigit = hasDigit || (c >= 48 /* Data.Character.Zero */ && c <= 57 /* Data.Character.Nine */); // Check for digits in the current word
        hasEmail = hasEmail || c === 64 /* Data.Character.At */; // Check for @ sign anywhere within the current word
        hasWhitespace = c === 9 /* Data.Character.Tab */ || c === 10 /* Data.Character.NewLine */ || c === 13 /* Data.Character.Return */ || c === 32 /* Data.Character.Blank */;
        // Process each word as an individual token to redact any sensitive information
        if (i === 0 || i === value.length - 1 || hasWhitespace) {
            // Performance optimization: Lazy load string -> array conversion only when required
            if (hasDigit || hasEmail) {
                if (array === null) {
                    array = value.split("" /* Data.Constant.Empty */);
                }
                // Work on a token at a time so we don't have to apply regex to a larger string
                let token = value.substring(spaceIndex + 1, hasWhitespace ? i : i + 1);
                // Check if unicode regex is supported, otherwise fallback to calling mask function on this token
                if (unicodeRegex && currencyRegex !== null) {
                    // Do not redact information if the token contains a currency symbol
                    token = token.match(currencyRegex) ? token : scrub(token, "\u25AA" /* Data.Constant.Letter */, "\u25AB" /* Data.Constant.Digit */);
                }
                else {
                    token = mask(token);
                }
                // Merge token back into array at the right place
                array.splice(spaceIndex + 1 - gap, token.length, token);
                gap += token.length - 1;
            }
            // Reset digit and email flags after every word boundary, except the beginning of string
            if (hasWhitespace) {
                hasDigit = false;
                hasEmail = false;
                spaceIndex = i;
            }
        }
    }
    return array ? array.join("" /* Data.Constant.Empty */) : value;
}

let state$b = null;
let buffer = null;
let update$2 = false;
function start$H() {
    update$2 = false;
    reset$s();
}
function reset$s() {
    // Baseline state holds the previous values - if it is updated in the current payload,
    // reset the state to current value after sending the previous state
    if (update$2) {
        state$b = { time: time(), event: 4 /* Event.Baseline */, data: {
                visible: buffer.visible,
                docWidth: buffer.docWidth,
                docHeight: buffer.docHeight,
                screenWidth: buffer.screenWidth,
                screenHeight: buffer.screenHeight,
                scrollX: buffer.scrollX,
                scrollY: buffer.scrollY,
                pointerX: buffer.pointerX,
                pointerY: buffer.pointerY,
                activityTime: buffer.activityTime,
                scrollTime: buffer.scrollTime,
                pointerTime: buffer.pointerTime,
                moveX: buffer.moveX,
                moveY: buffer.moveY,
                moveTime: buffer.moveTime,
                downX: buffer.downX,
                downY: buffer.downY,
                downTime: buffer.downTime,
                upX: buffer.upX,
                upY: buffer.upY,
                upTime: buffer.upTime,
                pointerPrevX: buffer.pointerPrevX,
                pointerPrevY: buffer.pointerPrevY,
                pointerPrevTime: buffer.pointerPrevTime,
            }
        };
    }
    buffer = buffer ? buffer : {
        visible: 1 /* BooleanFlag.True */,
        docWidth: 0,
        docHeight: 0,
        screenWidth: 0,
        screenHeight: 0,
        scrollX: 0,
        scrollY: 0,
        pointerX: 0,
        pointerY: 0,
        activityTime: 0,
        scrollTime: 0,
        pointerTime: undefined,
        moveX: undefined,
        moveY: undefined,
        moveTime: undefined,
        downX: undefined,
        downY: undefined,
        downTime: undefined,
        upX: undefined,
        upY: undefined,
        upTime: undefined,
        pointerPrevX: undefined,
        pointerPrevY: undefined,
        pointerPrevTime: undefined,
    };
}
function track$8(event, x, y, time) {
    switch (event) {
        case 8 /* Event.Document */:
            buffer.docWidth = x;
            buffer.docHeight = y;
            break;
        case 11 /* Event.Resize */:
            buffer.screenWidth = x;
            buffer.screenHeight = y;
            break;
        case 10 /* Event.Scroll */:
            buffer.scrollX = x;
            buffer.scrollY = y;
            buffer.scrollTime = time;
            break;
        case 12 /* Event.MouseMove */:
            buffer.moveX = x;
            buffer.moveY = y;
            buffer.moveTime = time;
            buffer.pointerPrevX = buffer.pointerX;
            buffer.pointerPrevY = buffer.pointerY;
            buffer.pointerPrevTime = buffer.pointerTime;
            buffer.pointerX = x;
            buffer.pointerY = y;
            buffer.pointerTime = time;
            break;
        case 13 /* Event.MouseDown */:
            buffer.downX = x;
            buffer.downY = y;
            buffer.downTime = time;
            buffer.pointerPrevX = buffer.pointerX;
            buffer.pointerPrevY = buffer.pointerY;
            buffer.pointerPrevTime = buffer.pointerTime;
            buffer.pointerX = x;
            buffer.pointerY = y;
            buffer.pointerTime = time;
            break;
        case 14 /* Event.MouseUp */:
            buffer.upX = x;
            buffer.upY = y;
            buffer.upTime = time;
            buffer.pointerPrevX = buffer.pointerX;
            buffer.pointerPrevY = buffer.pointerY;
            buffer.pointerPrevTime = buffer.pointerTime;
            buffer.pointerX = x;
            buffer.pointerY = y;
            buffer.pointerTime = time;
            break;
        default:
            buffer.pointerPrevX = buffer.pointerX;
            buffer.pointerPrevY = buffer.pointerY;
            buffer.pointerPrevTime = buffer.pointerTime;
            buffer.pointerX = x;
            buffer.pointerY = y;
            buffer.pointerTime = time;
            break;
    }
    update$2 = true;
}
function activity(t) {
    buffer.activityTime = t;
}
function visibility(t, visible) {
    buffer.visible = visible === "visible" ? 1 /* BooleanFlag.True */ : 0 /* BooleanFlag.False */;
    if (!buffer.visible) {
        activity(t);
    }
    update$2 = true;
}
function compute$e() {
    if (update$2) {
        encode$1(4 /* Event.Baseline */);
    }
}
function stop$E() {
    reset$s();
}

var baseline = /*#__PURE__*/Object.freeze({
    __proto__: null,
    activity: activity,
    compute: compute$e,
    reset: reset$s,
    start: start$H,
    get state () { return state$b; },
    stop: stop$E,
    track: track$8,
    visibility: visibility
});

let data$j = null;
// custom events allow 2 parameters or 1 parameter to be passed. If 2 are passed we
// consider it a key value pair. If only 1 is passed we only consider the event to have a value.
function event(a, b) {
    if (active() &&
        a &&
        typeof a === "string" /* Constant.String */ &&
        a.length < 255) {
        if (b && typeof b === "string" /* Constant.String */ && b.length < 255) {
            data$j = { key: a, value: b };
        }
        else {
            data$j = { value: a };
        }
        encode$1(24 /* Event.Custom */);
    }
}

let data$i = null;
let updates$3 = null;
function start$G() {
    data$i = {};
    updates$3 = {};
    count$1(5 /* Metric.InvokeCount */);
}
function stop$D() {
    data$i = {};
    updates$3 = {};
}
function count$1(metric) {
    if (!(metric in data$i)) {
        data$i[metric] = 0;
    }
    if (!(metric in updates$3)) {
        updates$3[metric] = 0;
    }
    data$i[metric]++;
    updates$3[metric]++;
}
function sum(metric, value) {
    if (value !== null) {
        if (!(metric in data$i)) {
            data$i[metric] = 0;
        }
        if (!(metric in updates$3)) {
            updates$3[metric] = 0;
        }
        data$i[metric] += value;
        updates$3[metric] += value;
    }
}
function max(metric, value) {
    // Ensure that we do not process null or NaN values
    if (value !== null && isNaN(value) === false) {
        if (!(metric in data$i)) {
            data$i[metric] = 0;
        }
        if (value > data$i[metric] || data$i[metric] === 0) {
            updates$3[metric] = value;
            data$i[metric] = value;
        }
    }
}
function compute$d() {
    encode$1(0 /* Event.Metric */);
}
function reset$r() {
    updates$3 = {};
}

function setTimeout(handler, timeout, event) {
    return window.setTimeout(measure(handler), timeout, event);
}
function clearTimeout(handle) {
    return window.clearTimeout(handle);
}

let data$h;
let last = 0;
let interval = 0;
let timeout$7 = null;
function start$F() {
    interval = 60000 /* Setting.PingInterval */;
    last = 0;
}
function reset$q() {
    if (timeout$7) {
        clearTimeout(timeout$7);
    }
    timeout$7 = setTimeout(ping, interval);
    last = time();
}
function ping() {
    let now = time();
    data$h = { gap: now - last };
    encode$1(25 /* Event.Ping */);
    if (data$h.gap < 300000 /* Setting.PingTimeout */) {
        timeout$7 = setTimeout(ping, interval);
    }
    else {
        suspend();
    }
}
function stop$C() {
    clearTimeout(timeout$7);
    last = 0;
    interval = 0;
}

var ping$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    get data () { return data$h; },
    reset: reset$q,
    start: start$F,
    stop: stop$C
});

let data$g = null;
function start$E() {
    data$g = {};
}
function stop$B() {
    data$g = {};
}
function track$7(event, time) {
    if (!(event in data$g)) {
        data$g[event] = [[time, 0]];
    }
    else {
        let e = data$g[event];
        let last = e[e.length - 1];
        // Add a new entry only if the new event occurs after configured interval
        // Otherwise, extend the duration of the previous entry
        if (time - last[0] > 100 /* Setting.SummaryInterval */) {
            data$g[event].push([time, 0]);
        }
        else {
            last[1] = time - last[0];
        }
    }
}
function compute$c() {
    encode$1(36 /* Event.Summary */);
}
function reset$p() {
    data$g = {};
}

var summary = /*#__PURE__*/Object.freeze({
    __proto__: null,
    compute: compute$c,
    get data () { return data$g; },
    reset: reset$p,
    start: start$E,
    stop: stop$B,
    track: track$7
});

const supported$1 = "CompressionStream" /* Constant.CompressionStream */ in window;
async function compress (input) {
    try {
        if (supported$1) {
            // Create a readable stream from given input string and
            // pipe it through text encoder and compression stream to gzip it
            const stream = new ReadableStream({ async start(controller) {
                    controller.enqueue(input);
                    controller.close();
                } }).pipeThrough(new TextEncoderStream()).pipeThrough(new window["CompressionStream" /* Constant.CompressionStream */]("gzip"));
            return new Uint8Array(await read(stream));
        }
    }
    catch ( /* do nothing */_a) { /* do nothing */ }
    return null;
}
async function read(stream) {
    const reader = stream.getReader();
    const chunks = [];
    let done = false;
    let value = [];
    while (!done) {
        ({ done, value } = await reader.read());
        if (done) {
            return chunks;
        }
        chunks.push(...value);
    }
    return chunks;
}

let data$f = null;
function start$D() {
    reset$o();
}
function set(variable, value) {
    let values = typeof value === "string" /* Constant.String */ ? [value] : value;
    log$2(variable, values);
}
async function identify(userId, sessionId = null, pageId = null, userHint = null) {
    let output = { userId: await sha256(userId), userHint: userHint || redact(userId) };
    // By default, hash custom userId using SHA256 algorithm on the client to preserve privacy
    log$2("userId" /* Constant.UserId */, [output.userId]);
    // Optional non-identifying name for the user
    // If name is not explicitly provided, we automatically generate a redacted version of the userId
    log$2("userHint" /* Constant.UserHint */, [output.userHint]);
    log$2("userType" /* Constant.UserType */, [detect(userId)]);
    // Log sessionId and pageId if provided
    if (sessionId) {
        log$2("sessionId" /* Constant.SessionId */, [sessionId]);
        output.sessionId = sessionId;
    }
    if (pageId) {
        log$2("pageId" /* Constant.PageId */, [pageId]);
        output.pageId = pageId;
    }
    return output;
}
function log$2(variable, value) {
    if (active() &&
        variable &&
        value &&
        typeof variable === "string" /* Constant.String */ &&
        variable.length < 255) {
        let validValues = variable in data$f ? data$f[variable] : [];
        for (let i = 0; i < value.length; i++) {
            if (typeof value[i] === "string" /* Constant.String */ && value[i].length < 255) {
                validValues.push(value[i]);
            }
        }
        data$f[variable] = validValues;
    }
}
function compute$b() {
    encode$1(34 /* Event.Variable */);
}
function reset$o() {
    data$f = {};
}
function stop$A() {
    reset$o();
}
function redact(input) {
    return input && input.length >= 5 /* Setting.WordLength */ ?
        `${input.substring(0, 2)}${scrub(input.substring(2), "*" /* Constant.Asterix */, "*" /* Constant.Asterix */)}` : scrub(input, "*" /* Constant.Asterix */, "*" /* Constant.Asterix */);
}
async function sha256(input) {
    try {
        if (crypto && input) {
            // Reference: https://developer.mozilla.org/en-US/docs/Web/API/SubtleCrypto/digest#converting_a_digest_to_a_hex_string
            const buffer = await crypto.subtle.digest("SHA-256" /* Constant.SHA256 */, new TextEncoder().encode(input));
            return Array.prototype.map.call(new Uint8Array(buffer), (x) => (('00' + x.toString(16)).slice(-2))).join('');
        }
        else {
            return "" /* Constant.Empty */;
        }
    }
    catch (_a) {
        return "" /* Constant.Empty */;
    }
}
function detect(input) {
    return input && input.indexOf("@" /* Constant.At */) > 0 ? "email" /* Constant.Email */ : "string" /* Constant.String */;
}

var variable = /*#__PURE__*/Object.freeze({
    __proto__: null,
    compute: compute$b,
    get data () { return data$f; },
    identify: identify,
    reset: reset$o,
    set: set,
    start: start$D,
    stop: stop$A
});

let data$e = {};
let keys = new Set();
let variables = {};
let selectors = {};
let hashes = {};
let validation = {};
function start$C() {
    reset$n();
}
// Input string is of the following form:
// EXTRACT 101|element { "1": ".class1", "2": "~window.a.b", "3": "!abc"}
// if element is present on the page it will set up event 101 to grab the contents of the class1 selector into component 1,
// the javascript evaluated contents of window.a.b into component 2,
// and the contents of Clarity's hash abc into component 3
function trigger$2(input) {
    try {
        var parts = input && input.length > 0 ? input.split(/ (.*)/) : ["" /* Constant.Empty */];
        var keyparts = parts[0].split(/\|(.*)/);
        var key = parseInt(keyparts[0]);
        var element = keyparts.length > 1 ? keyparts[1] : "" /* Constant.Empty */;
        var values = parts.length > 1 ? JSON.parse(parts[1]) : {};
        variables[key] = {};
        selectors[key] = {};
        hashes[key] = {};
        validation[key] = element;
        for (var v in values) {
            // values is a set of strings for proper JSON parsing, but it's more efficient 
            // to interact with them as numbers
            let id = parseInt(v);
            let value = values[v];
            let source = 2 /* ExtractSource.Text */;
            if (value.startsWith("~" /* Constant.Tilde */)) {
                source = 0 /* ExtractSource.Javascript */;
            }
            else if (value.startsWith("!" /* Constant.Bang */)) {
                source = 4 /* ExtractSource.Hash */;
            }
            switch (source) {
                case 0 /* ExtractSource.Javascript */:
                    let variable = value.slice(1);
                    variables[key][id] = parse$1(variable);
                    break;
                case 2 /* ExtractSource.Text */:
                    selectors[key][id] = value;
                    break;
                case 4 /* ExtractSource.Hash */:
                    let hash = value.slice(1);
                    hashes[key][id] = hash;
                    break;
            }
        }
    }
    catch (e) {
        log$1(8 /* Code.Config */, 1 /* Severity.Warning */, e ? e.name : null);
    }
}
function clone$1(v) {
    return JSON.parse(JSON.stringify(v));
}
function compute$a() {
    try {
        for (let v in variables) {
            let key = parseInt(v);
            if (validation[key] == "" /* Constant.Empty */ || document.querySelector(validation[key])) {
                let variableData = variables[key];
                for (let v in variableData) {
                    let variableKey = parseInt(v);
                    let value = str$1(evaluate(clone$1(variableData[variableKey])));
                    if (value) {
                        update$1(key, variableKey, value);
                    }
                }
                let selectorData = selectors[key];
                for (let s in selectorData) {
                    let shouldMask = false;
                    let selectorKey = parseInt(s);
                    let selector = selectorData[selectorKey];
                    if (selector.startsWith("@" /* Constant.At */)) {
                        shouldMask = true;
                        selector = selector.slice(1);
                    }
                    let nodes = document.querySelectorAll(selector);
                    if (nodes) {
                        let text = Array.from(nodes).map(e => e.textContent).join("<SEP>" /* Constant.Seperator */);
                        update$1(key, selectorKey, (shouldMask ? hash(text).trim() : text).slice(0, 10000 /* Setting.ExtractLimit */));
                    }
                }
                let hashData = hashes[key];
                for (let h in hashData) {
                    let hashKey = parseInt(h);
                    let content = hashText(hashData[hashKey]).trim().slice(0, 10000 /* Setting.ExtractLimit */);
                    update$1(key, hashKey, content);
                }
            }
        }
        if (keys.size > 0) {
            encode$1(40 /* Event.Extract */);
        }
    }
    catch (e) {
        log$1(5 /* Code.Selector */, 1 /* Severity.Warning */, e ? e.name : null);
    }
}
function reset$n() {
    keys.clear();
}
function update$1(key, subkey, value) {
    var update = false;
    if (!(key in data$e)) {
        data$e[key] = {};
        update = true;
    }
    if (!isEmpty(hashes[key])
        && (!(subkey in data$e[key]) || data$e[key][subkey] != value)) {
        update = true;
    }
    data$e[key][subkey] = value;
    if (update) {
        keys.add(key);
    }
    return;
}
function stop$z() {
    reset$n();
}
function parse$1(variable) {
    let syntax = [];
    let parts = variable.split("." /* Constant.Dot */);
    while (parts.length > 0) {
        let part = parts.shift();
        let arrayStart = part.indexOf("[" /* Constant.ArrayStart */);
        let conditionStart = part.indexOf("{" /* Constant.ConditionStart */);
        let conditionEnd = part.indexOf("}" /* Constant.ConditionEnd */);
        syntax.push({
            name: arrayStart > 0 ? part.slice(0, arrayStart) : (conditionStart > 0 ? part.slice(0, conditionStart) : part),
            type: arrayStart > 0 ? 1 /* Type.Array */ : (conditionStart > 0 ? 2 /* Type.Object */ : 3 /* Type.Simple */),
            condition: conditionStart > 0 ? part.slice(conditionStart + 1, conditionEnd) : null
        });
    }
    return syntax;
}
// The function below takes in a variable name in following format: "a.b.c" and safely evaluates its value in javascript context
// For instance, for a.b.c, it will first check window["a"]. If it exists, it will recursively look at: window["a"]["b"] and finally,
// return the value for window["a"]["b"]["c"].
function evaluate(variable, base = window) {
    if (variable.length == 0) {
        return base;
    }
    let part = variable.shift();
    let output;
    if (base && base[part.name]) {
        let obj = base[part.name];
        if (part.type !== 1 /* Type.Array */ && match(obj, part.condition)) {
            output = evaluate(variable, obj);
        }
        else if (Array.isArray(obj)) {
            let filtered = [];
            for (var value of obj) {
                if (match(value, part.condition)) {
                    let op = evaluate(variable, value);
                    if (op) {
                        filtered.push(op);
                    }
                }
            }
            output = filtered;
        }
        return output;
    }
    return null;
}
function str$1(input) {
    // Automatically trim string to max of Setting.ExtractLimit to avoid fetching long strings
    return input ? JSON.stringify(input).slice(0, 10000 /* Setting.ExtractLimit */) : input;
}
function match(base, condition) {
    if (condition) {
        let prop = condition.split(":");
        return prop.length > 1 ? base[prop[0]] == prop[1] : base[prop[0]];
    }
    return true;
}
function isEmpty(obj) {
    return Object.keys(obj).length == 0;
}

var extract = /*#__PURE__*/Object.freeze({
    __proto__: null,
    clone: clone$1,
    compute: compute$a,
    data: data$e,
    keys: keys,
    reset: reset$n,
    start: start$C,
    stop: stop$z,
    trigger: trigger$2,
    update: update$1
});

let signalCallback = null;
function signal(cb) {
    signalCallback = cb;
}
function parseSignals(signalsPayload) {
    try {
        const parsedSignals = JSON.parse(signalsPayload);
        return parsedSignals;
    }
    catch (_a) {
        return [];
    }
}
function signalsEvent(signalsPayload) {
    try {
        if (!signalCallback) {
            return;
        }
        const signals = parseSignals(signalsPayload);
        signals.forEach((signal) => {
            signalCallback(signal);
        });
    }
    catch (_a) {
        //do nothing
    }
}

const modules$1 = [baseline, dimension, variable, limit, summary, metadata$1, envelope$1, upload$1, ping$1, upgrade$1, extract];
function start$B() {
    // Metric needs to be initialized before we can start measuring. so metric is not wrapped in measure
    start$G();
    modules$1.forEach(x => measure(x.start)());
}
function stop$y() {
    // Stop modules in the reverse order of their initialization
    // The ordering below should respect inter-module dependency.
    // E.g. if upgrade depends on upload, then upgrade needs to end before upload.
    // Similarly, if upload depends on metadata, upload needs to end before metadata.
    modules$1.slice().reverse().forEach(x => measure(x.stop)());
    stop$D();
}
function compute$9() {
    compute$b();
    compute$e();
    compute$2();
    compute$d();
    compute$c();
    compute$3();
    compute$a();
}

let history$5 = [];
let data$d;
function start$A() {
    history$5 = [];
    max(26 /* Metric.Automation */, navigator.webdriver ? 1 /* BooleanFlag.True */ : 0 /* BooleanFlag.False */);
    try {
        // some sites (unintentionally) overwrite the window.self property, so we also check for the main window object
        max(31 /* Metric.Iframed */, window.top == window.self || window.top == window ? 1 /* IframeStatus.TopFrame */ : 2 /* IframeStatus.Iframe */);
    }
    catch (ex) {
        max(31 /* Metric.Iframed */, 0 /* IframeStatus.Unknown */);
    }
}
function check$4(id, target, input) {
    // Compute hash for fraud detection, if enabled. Hash is computed only if input meets the minimum length criteria
    if (config$2.fraud && id !== null && input && input.length >= 5 /* Setting.WordLength */) {
        data$d = { id, target, checksum: hash(input, 28 /* Setting.ChecksumPrecision */) };
        // Only encode this event if we haven't already reported this hash
        if (history$5.indexOf(data$d.checksum) < 0) {
            history$5.push(data$d.checksum);
            encode$2(41 /* Event.Fraud */);
        }
    }
}

let state$a = [];
function start$z() {
    reset$m();
}
function observe$c(root) {
    bind(root, "change", recompute$8, true);
}
function recompute$8(evt) {
    recompute$8.dn = 5 /* FunctionNames.ChangeRecompute */;
    let element = target(evt);
    if (element) {
        let value = element.value;
        let checksum = value && value.length >= 5 /* Setting.WordLength */ && config$2.fraud && "password,secret,pass,social,ssn,code,hidden" /* Mask.Exclude */.indexOf(element.type) === -1 ? hash(value, 28 /* Setting.ChecksumPrecision */) : "" /* Constant.Empty */;
        state$a.push({ time: time(evt), event: 42 /* Event.Change */, data: { target: target(evt), type: element.type, value, checksum } });
        schedule(encode$3.bind(this, 42 /* Event.Change */));
    }
}
function reset$m() {
    state$a = [];
}
function stop$x() {
    reset$m();
}

function offset(element) {
    let output = { x: 0, y: 0 };
    // Walk up the chain to ensure we compute offset distance correctly
    // In case where we may have nested IFRAMEs, we keep walking up until we get to the top most parent page
    if (element && element.offsetParent) {
        do {
            let parent = element.offsetParent;
            let frame = parent === null ? iframe(element.ownerDocument) : null;
            output.x += element.offsetLeft;
            output.y += element.offsetTop;
            element = frame ? frame : parent;
        } while (element);
    }
    return output;
}

const UserInputTags = ["input", "textarea", "radio", "button", "canvas", "select"];
let state$9 = [];
function start$y() {
    reset$l();
}
function observe$b(root) {
    bind(root, "click", handler$3.bind(this, 9 /* Event.Click */, root), true);
}
function handler$3(event, root, evt) {
    handler$3.dn = 6 /* FunctionNames.ClickHandler */;
    let frame = iframe(root);
    let d = frame ? frame.contentDocument.documentElement : document.documentElement;
    let x = "pageX" in evt ? Math.round(evt.pageX) : ("clientX" in evt ? Math.round(evt["clientX"] + d.scrollLeft) : null);
    let y = "pageY" in evt ? Math.round(evt.pageY) : ("clientY" in evt ? Math.round(evt["clientY"] + d.scrollTop) : null);
    // In case of iframe, we adjust (x,y) to be relative to top parent's origin
    if (frame) {
        let distance = offset(frame);
        x = x ? x + Math.round(distance.x) : x;
        y = y ? y + Math.round(distance.y) : y;
    }
    let t = target(evt);
    // Find nearest anchor tag (<a/>) parent if current target node is part of one
    // If present, we use the returned link element to populate text and link properties below
    let a = link(t);
    // Get layout rectangle for the target element
    let l = layout$1(t);
    // Reference: https://developer.mozilla.org/en-US/docs/Web/API/UIEvent/detail
    // This property helps differentiate between a keyboard navigation vs. pointer click
    // In case of a keyboard navigation, we use center of target element as (x,y)
    if (evt.detail === 0 && l) {
        x = Math.round(l.x + (l.w / 2));
        y = Math.round(l.y + (l.h / 2));
    }
    let eX = l ? Math.max(Math.floor(((x - l.x) / l.w) * 32767 /* Setting.ClickPrecision */), 0) : 0;
    let eY = l ? Math.max(Math.floor(((y - l.y) / l.h) * 32767 /* Setting.ClickPrecision */), 0) : 0;
    // Check for null values before processing this event
    if (x !== null && y !== null) {
        const textInfo = text(t);
        state$9.push({
            time: time(evt), event, data: {
                target: t,
                x,
                y,
                eX,
                eY,
                button: evt.button,
                reaction: reaction(t),
                context: context(a),
                text: textInfo.text,
                link: a ? a.href : null,
                hash: null,
                trust: evt.isTrusted ? 1 /* BooleanFlag.True */ : 0 /* BooleanFlag.False */,
                isFullText: textInfo.isFullText,
            }
        });
        schedule(encode$3.bind(this, event));
    }
}
function link(node) {
    while (node && node !== document) {
        if (node.nodeType === Node.ELEMENT_NODE) {
            let element = node;
            if (element.tagName === "A") {
                return element;
            }
        }
        node = node.parentNode;
    }
    return null;
}
function text(element) {
    let output = null;
    let isFullText = false;
    if (element) {
        // Grab text using "textContent" for most HTMLElements, however, use "value" for HTMLInputElements and "alt" for HTMLImageElement.
        let t = element.textContent || String(element.value || '') || element.alt;
        if (t) {
            // Replace multiple occurrence of space characters with a single white space
            // Also, trim any spaces at the beginning or at the end of string
            const trimmedText = t.replace(/\s+/g, " " /* Constant.Space */).trim();
            // Finally, send only first few characters as specified by the Setting
            output = trimmedText.substring(0, 25 /* Setting.ClickText */);
            isFullText = output.length === trimmedText.length;
        }
    }
    return { text: output, isFullText: isFullText ? 1 /* BooleanFlag.True */ : 0 /* BooleanFlag.False */ };
}
function reaction(element) {
    if (element.nodeType === Node.ELEMENT_NODE) {
        let tag = element.tagName.toLowerCase();
        if (UserInputTags.indexOf(tag) >= 0) {
            return 0 /* BooleanFlag.False */;
        }
    }
    return 1 /* BooleanFlag.True */;
}
function layout$1(element) {
    let box = null;
    let de = document.documentElement;
    if (typeof element.getBoundingClientRect === "function") {
        // getBoundingClientRect returns rectangle relative positioning to viewport
        let rect = element.getBoundingClientRect();
        if (rect && rect.width > 0 && rect.height > 0) {
            // Add viewport's scroll position to rectangle to get position relative to document origin
            // Also: using Math.floor() instead of Math.round() because in Edge,
            // getBoundingClientRect returns partial pixel values (e.g. 162.5px) and Chrome already
            // floors the value (e.g. 162px). This keeps consistent behavior across browsers.
            box = {
                x: Math.floor(rect.left + ("pageXOffset" in window ? window.pageXOffset : de.scrollLeft)),
                y: Math.floor(rect.top + ("pageYOffset" in window ? window.pageYOffset : de.scrollTop)),
                w: Math.floor(rect.width),
                h: Math.floor(rect.height)
            };
        }
    }
    return box;
}
function context(a) {
    if (a && a.hasAttribute("target" /* Constant.Target */)) {
        switch (a.getAttribute("target" /* Constant.Target */)) {
            case "_blank" /* Constant.Blank */: return 1 /* BrowsingContext.Blank */;
            case "_parent" /* Constant.Parent */: return 2 /* BrowsingContext.Parent */;
            case "_top" /* Constant.Top */: return 3 /* BrowsingContext.Top */;
        }
    }
    return 0 /* BrowsingContext.Self */;
}
function reset$l() {
    state$9 = [];
}
function stop$w() {
    reset$l();
}

let state$8 = [];
function start$x() {
    reset$k();
}
function observe$a(root) {
    bind(root, "cut", recompute$7.bind(this, 0 /* Clipboard.Cut */), true);
    bind(root, "copy", recompute$7.bind(this, 1 /* Clipboard.Copy */), true);
    bind(root, "paste", recompute$7.bind(this, 2 /* Clipboard.Paste */), true);
}
function recompute$7(action, evt) {
    recompute$7.dn = 7 /* FunctionNames.ClipboardRecompute */;
    state$8.push({ time: time(evt), event: 38 /* Event.Clipboard */, data: { target: target(evt), action } });
    schedule(encode$3.bind(this, 38 /* Event.Clipboard */));
}
function reset$k() {
    state$8 = [];
}
function stop$v() {
    reset$k();
}

let timeout$6 = null;
let state$7 = [];
function start$w() {
    reset$j();
}
function observe$9(root) {
    bind(root, "input", recompute$6, true);
}
function recompute$6(evt) {
    recompute$6.dn = 9 /* FunctionNames.InputRecompute */;
    let input = target(evt);
    let value = get(input);
    if (input && input.type && value) {
        let v = input.value;
        let t = input.type;
        switch (input.type) {
            case "radio":
            case "checkbox":
                v = input.checked ? "true" : "false";
                break;
        }
        let data = { target: input, value: v, type: t };
        // If last entry in the queue is for the same target node as the current one, remove it so we can later swap it with current data.
        if (state$7.length > 0 && (state$7[state$7.length - 1].data.target === data.target)) {
            state$7.pop();
        }
        state$7.push({ time: time(evt), event: 27 /* Event.Input */, data });
        clearTimeout(timeout$6);
        timeout$6 = setTimeout(process$7, 1000 /* Setting.InputLookAhead */, 27 /* Event.Input */);
    }
}
function process$7(event) {
    schedule(encode$3.bind(this, event));
}
function reset$j() {
    state$7 = [];
}
function stop$u() {
    clearTimeout(timeout$6);
    reset$j();
}

let state$6 = [];
let timeout$5 = null;
let hasPrimaryTouch = false;
let primaryTouchId = 0;
const activeTouchPointIds = new Set();
function start$v() {
    reset$i();
}
function observe$8(root) {
    bind(root, "mousedown", mouse.bind(this, 13 /* Event.MouseDown */, root), true);
    bind(root, "mouseup", mouse.bind(this, 14 /* Event.MouseUp */, root), true);
    bind(root, "mousemove", mouse.bind(this, 12 /* Event.MouseMove */, root), true);
    bind(root, "wheel", mouse.bind(this, 15 /* Event.MouseWheel */, root), true);
    bind(root, "dblclick", mouse.bind(this, 16 /* Event.DoubleClick */, root), true);
    bind(root, "touchstart", touch.bind(this, 17 /* Event.TouchStart */, root), true);
    bind(root, "touchend", touch.bind(this, 18 /* Event.TouchEnd */, root), true);
    bind(root, "touchmove", touch.bind(this, 19 /* Event.TouchMove */, root), true);
    bind(root, "touchcancel", touch.bind(this, 20 /* Event.TouchCancel */, root), true);
}
function mouse(event, root, evt) {
    mouse.dn = 10 /* FunctionNames.PointerMouse */;
    let frame = iframe(root);
    let d = frame ? frame.contentDocument.documentElement : document.documentElement;
    let x = "pageX" in evt ? Math.round(evt.pageX) : ("clientX" in evt ? Math.round(evt["clientX"] + d.scrollLeft) : null);
    let y = "pageY" in evt ? Math.round(evt.pageY) : ("clientY" in evt ? Math.round(evt["clientY"] + d.scrollTop) : null);
    // In case of iframe, we adjust (x,y) to be relative to top parent's origin
    if (frame) {
        let distance = offset(frame);
        x = x ? x + Math.round(distance.x) : x;
        y = y ? y + Math.round(distance.y) : y;
    }
    // Check for null values before processing this event
    if (x !== null && y !== null) {
        handler$2({ time: time(evt), event, data: { target: target(evt), x, y } });
    }
}
function touch(event, root, evt) {
    touch.dn = 11 /* FunctionNames.PointerTouch */;
    let frame = iframe(root);
    let d = frame ? frame.contentDocument.documentElement : document.documentElement;
    let touches = evt.changedTouches;
    let t = time(evt);
    if (touches) {
        for (let i = 0; i < touches.length; i++) {
            let entry = touches[i];
            let x = "clientX" in entry ? Math.round(entry["clientX"] + d.scrollLeft) : null;
            let y = "clientY" in entry ? Math.round(entry["clientY"] + d.scrollTop) : null;
            x = x && frame ? x + Math.round(frame.offsetLeft) : x;
            y = y && frame ? y + Math.round(frame.offsetTop) : y;
            // We cannot rely on identifier to determine primary touch as its value doesn't always start with 0.
            // Safari/Webkit uses the address of the UITouch object as the identifier value for each touch point.
            const id = "identifier" in entry ? entry["identifier"] : undefined;
            switch (event) {
                case 17 /* Event.TouchStart */:
                    if (activeTouchPointIds.size === 0) {
                        // Track presence of primary touch separately to handle scenarios when same id is repeated
                        hasPrimaryTouch = true;
                        primaryTouchId = id;
                    }
                    activeTouchPointIds.add(id);
                    break;
                case 18 /* Event.TouchEnd */:
                case 20 /* Event.TouchCancel */:
                    activeTouchPointIds.delete(id);
                    break;
            }
            const isPrimary = hasPrimaryTouch && primaryTouchId === id;
            // Check for null values before processing this event
            if (x !== null && y !== null) {
                handler$2({ time: t, event, data: { target: target(evt), x, y, id, isPrimary } });
            }
            // Reset primary touch point id once touch event ends
            if (event === 20 /* Event.TouchCancel */ || event === 18 /* Event.TouchEnd */) {
                if (primaryTouchId === id) {
                    hasPrimaryTouch = false;
                }
            }
        }
    }
}
function handler$2(current) {
    switch (current.event) {
        case 12 /* Event.MouseMove */:
        case 15 /* Event.MouseWheel */:
        case 19 /* Event.TouchMove */:
            let length = state$6.length;
            let last = length > 1 ? state$6[length - 2] : null;
            if (last && similar$1(last, current)) {
                state$6.pop();
            }
            state$6.push(current);
            clearTimeout(timeout$5);
            timeout$5 = setTimeout(process$6, 500 /* Setting.LookAhead */, current.event);
            break;
        default:
            state$6.push(current);
            process$6(current.event);
            break;
    }
}
function process$6(event) {
    schedule(encode$3.bind(this, event));
}
function reset$i() {
    state$6 = [];
}
function similar$1(last, current) {
    let dx = last.data.x - current.data.x;
    let dy = last.data.y - current.data.y;
    let distance = Math.sqrt(dx * dx + dy * dy);
    let gap = current.time - last.time;
    let match = current.data.target === last.data.target;
    return current.event === last.event && match && distance < 20 /* Setting.Distance */ && gap < 25 /* Setting.Interval */;
}
function stop$t() {
    clearTimeout(timeout$5);
    // Send out any pending pointer events in the pipeline
    if (state$6.length > 0) {
        process$6(state$6[state$6.length - 1].event);
    }
}

let data$c;
let timeout$4 = null;
let initialStateLogged = false;
function start$u() {
    initialStateLogged = false;
    bind(window, "resize", recompute$5);
    recompute$5();
}
function recompute$5() {
    recompute$5.dn = 12 /* FunctionNames.ResizeRecompute */;
    let de = document.documentElement;
    // window.innerWidth includes width of the scrollbar and is not a true representation of the viewport width.
    // Therefore, when possible, use documentElement's clientWidth property.
    data$c = {
        width: de && "clientWidth" in de ? Math.min(de.clientWidth, window.innerWidth) : window.innerWidth,
        height: de && "clientHeight" in de ? Math.min(de.clientHeight, window.innerHeight) : window.innerHeight,
    };
    if (initialStateLogged) {
        clearTimeout(timeout$4);
        timeout$4 = setTimeout(process$5, 500 /* Setting.LookAhead */, 11 /* Event.Resize */);
    }
    else {
        encode$3(11 /* Event.Resize */);
        initialStateLogged = true;
    }
}
function process$5(event) {
    schedule(encode$3.bind(this, event));
}
function reset$h() {
    data$c = null;
    clearTimeout(timeout$4);
}
function stop$s() {
    reset$h();
}

let state$5 = [];
let initialTop = null;
let initialBottom = null;
let timeout$3 = null;
function start$t() {
    state$5 = [];
    recompute$4();
}
function observe$7(root) {
    let frame = iframe(root);
    let node = frame ? frame.contentWindow : (root === document ? window : root);
    bind(node, "scroll", recompute$4, true);
}
function recompute$4(event = null) {
    recompute$4.dn = 13 /* FunctionNames.ScrollRecompute */;
    let w = window;
    let de = document.documentElement;
    let element = event ? target(event) : de;
    // If the target is a Document node, then identify corresponding documentElement and window for this document
    if (element && element.nodeType === Node.DOCUMENT_NODE) {
        let frame = iframe(element);
        w = frame ? frame.contentWindow : w;
        element = de = element.documentElement;
    }
    // Edge doesn't support scrollTop position on document.documentElement.
    // For cross browser compatibility, looking up pageYOffset on window if the scroll is on document.
    // And, if for some reason that is not available, fall back to looking up scrollTop on document.documentElement.
    let x = element === de && "pageXOffset" in w ? Math.round(w.pageXOffset) : Math.round(element.scrollLeft);
    let y = element === de && "pageYOffset" in w ? Math.round(w.pageYOffset) : Math.round(element.scrollTop);
    const width = window.innerWidth;
    const height = window.innerHeight;
    const xPosition = width / 3;
    const yOffset = width > height ? height * 0.15 : height * 0.2;
    const startYPosition = yOffset;
    const endYPosition = height - yOffset;
    const top = getPositionNode(xPosition, startYPosition);
    const bottom = getPositionNode(xPosition, endYPosition);
    let current = { time: time(event), event: 10 /* Event.Scroll */, data: { target: element, x, y, top, bottom } };
    // We don't send any scroll events if this is the first event and the current position is top (0,0)
    if ((event === null && x === 0 && y === 0) || (x === null || y === null)) {
        initialTop = top;
        initialBottom = bottom;
        return;
    }
    let length = state$5.length;
    let last = length > 1 ? state$5[length - 2] : null;
    if (last && similar(last, current)) {
        state$5.pop();
    }
    state$5.push(current);
    clearTimeout(timeout$3);
    timeout$3 = setTimeout(process$4, 500 /* Setting.LookAhead */, 10 /* Event.Scroll */);
}
function getPositionNode(x, y) {
    var _a, _b;
    let node;
    if ("caretPositionFromPoint" in document) {
        node = (_a = document.caretPositionFromPoint(x, y)) === null || _a === void 0 ? void 0 : _a.offsetNode;
    }
    else if ("caretRangeFromPoint" in document) {
        node = (_b = document.caretRangeFromPoint(x, y)) === null || _b === void 0 ? void 0 : _b.startContainer;
    }
    if (!node) {
        node = document.elementFromPoint(x, y);
    }
    if (node && node.nodeType === Node.TEXT_NODE) {
        node = node.parentNode;
    }
    return node;
}
function reset$g() {
    state$5 = [];
    initialTop = null;
    initialBottom = null;
}
function process$4(event) {
    schedule(encode$3.bind(this, event));
}
function similar(last, current) {
    let dx = last.data.x - current.data.x;
    let dy = last.data.y - current.data.y;
    return (dx * dx + dy * dy < 20 /* Setting.Distance */ * 20 /* Setting.Distance */) && (current.time - last.time < 25 /* Setting.Interval */);
}
function compute$8() {
    var _a, _b;
    compute$8.dn = 14 /* FunctionNames.ScrollCompute */;
    if (initialTop) {
        const top = metadata$2(initialTop, null);
        log(31 /* Dimension.InitialScrollTop */, (_a = top === null || top === void 0 ? void 0 : top.hash) === null || _a === void 0 ? void 0 : _a.join("." /* Constant.Dot */));
    }
    if (initialBottom) {
        const bottom = metadata$2(initialBottom, null);
        log(32 /* Dimension.InitialScrollBottom */, (_b = bottom === null || bottom === void 0 ? void 0 : bottom.hash) === null || _b === void 0 ? void 0 : _b.join("." /* Constant.Dot */));
    }
}
function stop$r() {
    clearTimeout(timeout$3);
    state$5 = [];
    initialTop = null;
    initialBottom = null;
}

let data$b = null;
let previous = null;
let timeout$2 = null;
function start$s() {
    reset$f();
}
function observe$6(root) {
    bind(root, "selectstart", recompute$3.bind(this, root), true);
    bind(root, "selectionchange", recompute$3.bind(this, root), true);
}
function recompute$3(root) {
    recompute$3.dn = 15 /* FunctionNames.SelectionRecompute */;
    let doc = root.nodeType === Node.DOCUMENT_NODE ? root : document;
    let current = doc.getSelection();
    // Bail out if we don't have a valid selection
    if (current === null) {
        return;
    }
    // Bail out if we got a valid selection but not valid nodes
    // In Edge, selectionchange gets fired even on interactions like right clicks and
    // can result in null anchorNode and focusNode if there was no previous selection on page
    // Also, ignore any selections that start and end at the exact same point
    if ((current.anchorNode === null && current.focusNode === null) ||
        (current.anchorNode === current.focusNode && current.anchorOffset === current.focusOffset)) {
        return;
    }
    let startNode = data$b.start ? data$b.start : null;
    if (previous !== null && data$b.start !== null && startNode !== current.anchorNode) {
        clearTimeout(timeout$2);
        process$3(21 /* Event.Selection */);
    }
    data$b = {
        start: current.anchorNode,
        startOffset: current.anchorOffset,
        end: current.focusNode,
        endOffset: current.focusOffset
    };
    previous = current;
    clearTimeout(timeout$2);
    timeout$2 = setTimeout(process$3, 500 /* Setting.LookAhead */, 21 /* Event.Selection */);
}
function process$3(event) {
    schedule(encode$3.bind(this, event));
}
function reset$f() {
    previous = null;
    data$b = { start: 0, startOffset: 0, end: 0, endOffset: 0 };
}
function stop$q() {
    reset$f();
    clearTimeout(timeout$2);
}

let state$4 = [];
function start$r() {
    reset$e();
}
function observe$5(root) {
    bind(root, "submit", recompute$2, true);
}
function recompute$2(evt) {
    recompute$2.dn = 16 /* FunctionNames.SubmitRecompute */;
    state$4.push({ time: time(evt), event: 39 /* Event.Submit */, data: { target: target(evt) } });
    schedule(encode$3.bind(this, 39 /* Event.Submit */));
}
function reset$e() {
    state$4 = [];
}
function stop$p() {
    reset$e();
}

let data$a;
function start$q() {
    bind(window, "pagehide", recompute$1);
}
function recompute$1(evt) {
    recompute$1.dn = 17 /* FunctionNames.UnloadRecompute */;
    data$a = { name: evt.type, persisted: evt.persisted ? 1 /* BooleanFlag.True */ : 0 /* BooleanFlag.False */ };
    encode$3(26 /* Event.Unload */, time(evt));
    stop();
}
function reset$d() {
    data$a = null;
}
function stop$o() {
    reset$d();
}

let data$9;
function start$p() {
    bind(document, "visibilitychange", recompute);
    recompute();
}
function recompute(evt = null) {
    recompute.dn = 18 /* FunctionNames.VisibilityRecompute */;
    data$9 = { visible: "visibilityState" in document ? document.visibilityState : "default" };
    encode$3(28 /* Event.Visibility */, time(evt));
}
function reset$c() {
    data$9 = null;
}
function stop$n() {
    reset$c();
}

function start$o() {
    start$o.dn = 8 /* FunctionNames.InteractionStart */;
    start$h();
    start$y();
    start$x();
    start$v();
    start$w();
    start$u();
    start$p();
    start$t();
    start$s();
    start$z();
    start$r();
    start$q();
}
function stop$m() {
    stop$f();
    stop$w();
    stop$v();
    stop$t();
    stop$u();
    stop$s();
    stop$n();
    stop$r();
    stop$q();
    stop$x();
    stop$p();
    stop$o();
}
function observe$4(root) {
    observe$7(root);
    // Only monitor following interactions if the root node is a document
    // In case of shadow DOM, following events automatically bubble up to the parent document.
    if (root.nodeType === Node.DOCUMENT_NODE) {
        observe$b(root);
        observe$a(root);
        observe$8(root);
        observe$9(root);
        observe$6(root);
        observe$c(root);
        observe$5(root);
    }
}

var interaction = /*#__PURE__*/Object.freeze({
    __proto__: null,
    observe: observe$4,
    start: start$o,
    stop: stop$m
});

// Following code takes an array of tokens and transforms it to optimize for repeating tokens and make it efficient to send over the wire
// The way it works is that it iterate over all tokens and checks if the current token was already seen in the tokens array so far
// If so, it replaces the token with its reference (index). This helps us save bytes by not repeating the same value twice.
// E.g. If tokens array is: ["hello", "world", "coding", "language", "world", "language", "example"]
// Then the resulting tokens array after following code execution would be: ["hello", "world", "coding", "language", [1, 3], "example"]
// Where [1,3] points to tokens[1] => "world" and tokens[3] => "language"
function tokenize (tokens) {
    let output = [];
    let lookup = {};
    let pointer = 0;
    let reference = null;
    for (let i = 0; i < tokens.length; i++) {
        // Only optimize for string values
        if (typeof tokens[i] === "string" /* Constant.String */) {
            let token = tokens[i];
            let index = lookup[token] || -1;
            if (index >= 0) {
                if (reference) {
                    reference.push(index);
                }
                else {
                    reference = [index];
                    output.push(reference);
                    pointer++;
                }
            }
            else {
                reference = null;
                output.push(token);
                lookup[token] = pointer++;
            }
        }
        else {
            // If the value is anything other than string, append it as it is to the output array
            // And, also increment the pointer to stay in sync with output array
            reference = null;
            output.push(tokens[i]);
            pointer++;
        }
    }
    return output;
}

let sheetUpdateState = [];
let sheetAdoptionState = [];
const styleSheetId = 'claritySheetId';
let styleSheetMap = {};
let styleTimeMap = {};
let documentNodes = [];
let createdSheetIds = [];
function proxyStyleRules$1(win) {
    if ((config$2.lean && config$2.lite) || win === null || win === undefined) {
        return;
    }
    win.clarityOverrides = win.clarityOverrides || {};
    if (win['CSSStyleSheet'] && win.CSSStyleSheet.prototype) {
        if (win.clarityOverrides.replace === undefined) {
            win.clarityOverrides.replace = win.CSSStyleSheet.prototype.replace;
            win.CSSStyleSheet.prototype.replace = function () {
                if (active()) {
                    // if we haven't seen this stylesheet on this page yet, wait until the checkDocumentStyles has found it
                    // and attached the sheet to a document. This way the timestamp of the style sheet creation will align
                    // to when it is used in the document rather than potentially being misaligned during the traverse process.
                    if (createdSheetIds.indexOf(this[styleSheetId]) > -1) {
                        trackStyleChange(time(), this[styleSheetId], 1 /* StyleSheetOperation.Replace */, arguments[0]);
                    }
                }
                return win.clarityOverrides.replace.apply(this, arguments);
            };
        }
        if (win.clarityOverrides.replaceSync === undefined) {
            win.clarityOverrides.replaceSync = win.CSSStyleSheet.prototype.replaceSync;
            win.CSSStyleSheet.prototype.replaceSync = function () {
                if (active()) {
                    // if we haven't seen this stylesheet on this page yet, wait until the checkDocumentStyles has found it
                    // and attached the sheet to a document. This way the timestamp of the style sheet creation will align
                    // to when it is used in the document rather than potentially being misaligned during the traverse process.
                    if (createdSheetIds.indexOf(this[styleSheetId]) > -1) {
                        trackStyleChange(time(), this[styleSheetId], 2 /* StyleSheetOperation.ReplaceSync */, arguments[0]);
                    }
                }
                return win.clarityOverrides.replaceSync.apply(this, arguments);
            };
        }
    }
}
function start$n() {
    proxyStyleRules$1(window);
}
function checkDocumentStyles(documentNode, timestamp) {
    if (config$2.lean && config$2.lite) {
        return;
    }
    if (documentNodes.indexOf(documentNode) === -1) {
        documentNodes.push(documentNode);
        if (documentNode.defaultView) {
            proxyStyleRules$1(documentNode.defaultView);
        }
    }
    timestamp = timestamp || time();
    if (!(documentNode === null || documentNode === void 0 ? void 0 : documentNode.adoptedStyleSheets)) {
        // if we don't have adoptedStyledSheets on the Node passed to us, we can short circuit.
        return;
    }
    let currentStyleSheets = [];
    for (var styleSheet of documentNode.adoptedStyleSheets) {
        // If we haven't seen this style sheet on this page yet, we create a reference to it for the visualizer.
        // For SPA or times in which Clarity restarts on a given page, our visualizer would lose context
        // on the previously created style sheet for page N-1.
        // Then we synthetically call replaceSync with its contents to bootstrap it
        if (!styleSheet[styleSheetId] || createdSheetIds.indexOf(styleSheet[styleSheetId]) === -1) {
            styleSheet[styleSheetId] = shortid();
            createdSheetIds.push(styleSheet[styleSheetId]);
            trackStyleChange(timestamp, styleSheet[styleSheetId], 0 /* StyleSheetOperation.Create */);
            trackStyleChange(timestamp, styleSheet[styleSheetId], 2 /* StyleSheetOperation.ReplaceSync */, getCssRules(styleSheet));
        }
        currentStyleSheets.push(styleSheet[styleSheetId]);
    }
    let documentId = getId(documentNode, true);
    if (!styleSheetMap[documentId]) {
        styleSheetMap[documentId] = [];
    }
    if (!arraysEqual(currentStyleSheets, styleSheetMap[documentId])) {
        // Using -1 to signify the root document node as we don't track that as part of our nodeMap
        trackStyleAdoption(timestamp, documentNode == document ? -1 : getId(documentNode), 3 /* StyleSheetOperation.SetAdoptedStyles */, currentStyleSheets);
        styleSheetMap[documentId] = currentStyleSheets;
        styleTimeMap[documentId] = timestamp;
    }
}
function compute$7() {
    for (var documentNode of documentNodes) {
        var docId = documentNode == document ? -1 : getId(documentNode);
        let ts = docId in styleTimeMap ? styleTimeMap[docId] : null;
        checkDocumentStyles(document, ts);
    }
}
function reset$b() {
    sheetAdoptionState = [];
    sheetUpdateState = [];
}
function stop$l() {
    styleSheetMap = {};
    styleTimeMap = {};
    documentNodes = [];
    createdSheetIds = [];
    reset$b();
}
function trackStyleChange(time, id, operation, cssRules) {
    sheetUpdateState.push({
        time,
        event: 46 /* Event.StyleSheetUpdate */,
        data: {
            id,
            operation,
            cssRules
        }
    });
    encode$4(46 /* Event.StyleSheetUpdate */);
}
function trackStyleAdoption(time, id, operation, newIds) {
    sheetAdoptionState.push({
        time,
        event: 45 /* Event.StyleSheetAdoption */,
        data: {
            id,
            operation,
            newIds
        }
    });
    encode$4(45 /* Event.StyleSheetAdoption */);
}
function arraysEqual(a, b) {
    if (a.length !== b.length) {
        return false;
    }
    return a.every((value, index) => value === b[index]);
}

let state$3 = [];
let elementAnimate = null;
let animationPlay = null;
let animationPause = null;
let animationCommitStyles = null;
let animationCancel = null;
let animationFinish = null;
const animationId = 'clarityAnimationId';
const operationCount = 'clarityOperationCount';
const maxOperations = 20;
function start$m() {
    if (window["Animation"] &&
        window["Animation"].prototype &&
        window["KeyframeEffect"] &&
        window["KeyframeEffect"].prototype &&
        window["KeyframeEffect"].prototype.getKeyframes &&
        window["KeyframeEffect"].prototype.getTiming) {
        reset$a();
        overrideAnimationHelper(animationPlay, "play");
        overrideAnimationHelper(animationPause, "pause");
        overrideAnimationHelper(animationCommitStyles, "commitStyles");
        overrideAnimationHelper(animationCancel, "cancel");
        overrideAnimationHelper(animationFinish, "finish");
        if (elementAnimate === null) {
            elementAnimate = Element.prototype.animate;
            Element.prototype.animate = function () {
                var createdAnimation = elementAnimate.apply(this, arguments);
                trackAnimationOperation(createdAnimation, "play");
                return createdAnimation;
            };
        }
        if (document.getAnimations) {
            for (var animation of document.getAnimations()) {
                if (animation.playState === "finished") {
                    trackAnimationOperation(animation, "finish");
                }
                else if (animation.playState === "paused" || animation.playState === "idle") {
                    trackAnimationOperation(animation, "pause");
                }
                else if (animation.playState === "running") {
                    trackAnimationOperation(animation, "play");
                }
            }
        }
    }
}
function reset$a() {
    state$3 = [];
}
function track$6(time, id, operation, keyFrames, timing, targetId, timeline) {
    state$3.push({
        time,
        event: 44 /* Event.Animation */,
        data: {
            id,
            operation,
            keyFrames,
            timing,
            targetId,
            timeline
        }
    });
    encode$4(44 /* Event.Animation */);
}
function stop$k() {
    reset$a();
}
function overrideAnimationHelper(functionToOverride, name) {
    if (functionToOverride === null) {
        functionToOverride = Animation.prototype[name];
        Animation.prototype[name] = function () {
            trackAnimationOperation(this, name);
            return functionToOverride.apply(this, arguments);
        };
    }
}
function trackAnimationOperation(animation, name) {
    if (active()) {
        let effect = animation.effect;
        let target = getId(effect.target);
        if (target !== null && effect.getKeyframes && effect.getTiming) {
            if (!animation[animationId]) {
                animation[animationId] = shortid();
                animation[operationCount] = 0;
                let keyframes = effect.getKeyframes();
                let timing = effect.getTiming();
                track$6(time(), animation[animationId], 0 /* AnimationOperation.Create */, JSON.stringify(keyframes), JSON.stringify(timing), target);
            }
            if (animation[operationCount]++ < maxOperations) {
                let operation = null;
                switch (name) {
                    case "play":
                        operation = 1 /* AnimationOperation.Play */;
                        break;
                    case "pause":
                        operation = 2 /* AnimationOperation.Pause */;
                        break;
                    case "cancel":
                        operation = 3 /* AnimationOperation.Cancel */;
                        break;
                    case "finish":
                        operation = 4 /* AnimationOperation.Finish */;
                        break;
                    case "commitStyles":
                        operation = 5 /* AnimationOperation.CommitStyles */;
                        break;
                }
                if (operation) {
                    track$6(time(), animation[animationId], operation);
                }
            }
        }
    }
}

async function encode$4 (type, timer = null, ts = null) {
    let eventTime = ts || time();
    let tokens = [eventTime, type];
    switch (type) {
        case 8 /* Event.Document */:
            let d = data$8;
            tokens.push(d.width);
            tokens.push(d.height);
            track$8(type, d.width, d.height);
            queue(tokens);
            break;
        case 7 /* Event.Region */:
            for (let r of state$2) {
                tokens = [r.time, 7 /* Event.Region */];
                tokens.push(r.data.id);
                tokens.push(r.data.interaction);
                tokens.push(r.data.visibility);
                tokens.push(r.data.name);
                queue(tokens);
            }
            reset$6();
            break;
        case 45 /* Event.StyleSheetAdoption */:
        case 46 /* Event.StyleSheetUpdate */:
            for (let entry of sheetAdoptionState) {
                tokens = [entry.time, entry.event];
                tokens.push(entry.data.id);
                tokens.push(entry.data.operation);
                tokens.push(entry.data.newIds);
                queue(tokens);
            }
            for (let entry of sheetUpdateState) {
                tokens = [entry.time, entry.event];
                tokens.push(entry.data.id);
                tokens.push(entry.data.operation);
                tokens.push(entry.data.cssRules);
                queue(tokens);
            }
            reset$b();
            break;
        case 44 /* Event.Animation */:
            for (let entry of state$3) {
                tokens = [entry.time, entry.event];
                tokens.push(entry.data.id);
                tokens.push(entry.data.operation);
                tokens.push(entry.data.keyFrames);
                tokens.push(entry.data.timing);
                tokens.push(entry.data.timeline);
                tokens.push(entry.data.targetId);
                queue(tokens);
            }
            reset$a();
            break;
        case 5 /* Event.Discover */:
        case 6 /* Event.Mutation */:
            // Check if we are operating within the context of the current page
            if (state(timer) === 2 /* Task.Stop */) {
                break;
            }
            let values = updates$2();
            // Only encode and queue DOM updates if we have valid updates to report back
            if (values.length > 0) {
                for (let value of values) {
                    let state$1 = state(timer);
                    if (state$1 === 0 /* Task.Wait */) {
                        state$1 = await suspend$1(timer);
                    }
                    if (state$1 === 2 /* Task.Stop */) {
                        break;
                    }
                    let data = value.data;
                    let active = value.metadata.active;
                    let suspend = value.metadata.suspend;
                    let privacy = value.metadata.privacy;
                    let mangle = shouldMangle(value);
                    let keys = active ? ["tag", "attributes", "value"] : ["tag"];
                    for (let key of keys) {
                        // we check for data[key] === '' because we want to encode empty strings as well, especially for value - which if skipped can cause our decoder to assume the final
                        // attribute was the value for the node
                        if (data[key] || data[key] === '') {
                            switch (key) {
                                case "tag":
                                    let box = size(value);
                                    let factor = mangle ? -1 : 1;
                                    tokens.push(value.id * factor);
                                    if (value.parent && active) {
                                        tokens.push(value.parent);
                                        if (value.previous) {
                                            tokens.push(value.previous);
                                        }
                                    }
                                    tokens.push(suspend ? "*M" /* Constant.SuspendMutationTag */ : data[key]);
                                    if (box && box.length === 2) {
                                        tokens.push(`${"#" /* Constant.Hash */}${str(box[0])}.${str(box[1])}`);
                                    }
                                    break;
                                case "attributes":
                                    for (let attr in data[key]) {
                                        if (data[key][attr] !== undefined) {
                                            tokens.push(attribute(attr, data[key][attr], privacy));
                                        }
                                    }
                                    break;
                                case "value":
                                    check$4(value.metadata.fraud, value.id, data[key]);
                                    tokens.push(text$1(data[key], data.tag, privacy, mangle));
                                    break;
                            }
                        }
                    }
                }
                if (type === 6 /* Event.Mutation */) {
                    activity(eventTime);
                }
                queue(tokenize(tokens), !config$2.lean);
            }
            break;
    }
}
function shouldMangle(value) {
    let privacy = value.metadata.privacy;
    return value.data.tag === "*T" /* Constant.TextTag */ && !(privacy === 0 /* Privacy.None */ || privacy === 1 /* Privacy.Sensitive */);
}
function size(value) {
    if (value.metadata.size !== null && value.metadata.size.length === 0) {
        let img = getNode(value.id);
        if (img) {
            return [Math.floor(img.offsetWidth * 100 /* Setting.BoxPrecision */), Math.floor(img.offsetHeight * 100 /* Setting.BoxPrecision */)];
        }
    }
    return value.metadata.size;
}
function str(input) {
    return input.toString(36);
}
function attribute(key, value, privacy) {
    return `${key}=${text$1(value, key.indexOf("data-" /* Constant.DataAttribute */) === 0 ? "data-" /* Constant.DataAttribute */ : key, privacy)}`;
}

let data$8;
function reset$9() {
    data$8 = null;
}
function start$l() {
    reset$9();
    compute$6();
}
function compute$6() {
    compute$6.dn = 19 /* FunctionNames.DocumentCompute */;
    let body = document.body;
    let d = document.documentElement;
    let bodyClientWidth = body ? body.clientWidth : null;
    let bodyScrollWidth = body ? body.scrollWidth : null;
    let bodyOffsetWidth = body ? body.offsetWidth : null;
    let documentClientWidth = d ? d.clientWidth : null;
    let documentScrollWidth = d ? d.scrollWidth : null;
    let documentOffsetWidth = d ? d.offsetWidth : null;
    let width = Math.max(bodyClientWidth, bodyScrollWidth, bodyOffsetWidth, documentClientWidth, documentScrollWidth, documentOffsetWidth);
    let bodyClientHeight = body ? body.clientHeight : null;
    let bodyScrollHeight = body ? body.scrollHeight : null;
    let bodyOffsetHeight = body ? body.offsetHeight : null;
    let documentClientHeight = d ? d.clientHeight : null;
    let documentScrollHeight = d ? d.scrollHeight : null;
    let documentOffsetHeight = d ? d.offsetHeight : null;
    let height = Math.max(bodyClientHeight, bodyScrollHeight, bodyOffsetHeight, documentClientHeight, documentScrollHeight, documentOffsetHeight);
    // Check that width or height has changed from before, and also that width & height are not null values
    if ((data$8 === null || width !== data$8.width || height !== data$8.height) && width !== null && height !== null) {
        data$8 = { width, height };
        encode$4(8 /* Event.Document */);
    }
}
function stop$j() {
    reset$9();
}

async function traverse (root, timer, source, timestamp) {
    let queue = [root];
    while (queue.length > 0) {
        let entry = queue.shift();
        let next = entry.firstChild;
        while (next) {
            queue.push(next);
            next = next.nextSibling;
        }
        // Check the status of current task to see if we should yield before continuing
        let state$1 = state(timer);
        if (state$1 === 0 /* Task.Wait */) {
            state$1 = await suspend$1(timer);
        }
        if (state$1 === 2 /* Task.Stop */) {
            break;
        }
        // Check if processing a node gives us a pointer to one of its sub nodes for traversal
        // E.g. an element node may give us a pointer to traverse shadowDom if shadowRoot property is set
        // Or, an iframe from the same origin could give a pointer to it's document for traversing contents of iframe.
        let subnode = processNode(entry, source, timestamp);
        if (subnode) {
            queue.push(subnode);
        }
    }
}

let observers = new Set();
let mutations = [];
let throttledMutations = {};
let queue$2 = [];
let timeout$1 = null;
let throttleDelay = null;
let activePeriod = null;
let history$4 = {};
let observedNodes = new WeakMap();
// We ignore mutations if these attributes are updated
const IGNORED_ATTRIBUTES = ["data-google-query-id", "data-load-complete", "data-google-container-id"];
function start$k() {
    start$k.dn = 21 /* FunctionNames.MutationStart */;
    observers = new Set();
    queue$2 = [];
    timeout$1 = null;
    activePeriod = 0;
    history$4 = {};
    observedNodes = new WeakMap();
    proxyStyleRules(window);
}
function observe$3(node) {
    // Create a new observer for every time a new DOM tree (e.g. root document or shadowdom root) is discovered on the page
    // In the case of shadow dom, any mutations that happen within the shadow dom are not bubbled up to the host document
    // For this reason, we need to wire up mutations every time we see a new shadow dom.
    // Also, wrap it inside a try / catch. In certain browsers (e.g. legacy Edge), observer on shadow dom can throw errors
    try {
        let m = api("MutationObserver" /* Constant.MutationObserver */);
        let observer = m in window ? new window[m](measure(handle$1)) : null;
        if (observer) {
            observer.observe(node, { attributes: true, childList: true, characterData: true, subtree: true });
            observedNodes.set(node, observer);
            observers.add(observer);
        }
        if (node['defaultView']) {
            proxyStyleRules(node['defaultView']);
        }
    }
    catch (e) {
        log$1(2 /* Code.MutationObserver */, 0 /* Severity.Info */, e ? e.name : null);
    }
}
function monitor(frame) {
    // Bind to iframe's onload event so we get notified anytime there's an update to iframe content.
    // This includes cases where iframe location is updated without explicitly updating src attribute
    // E.g. iframe.contentWindow.location.href = "new-location";
    if (has$1(frame) === false) {
        bind(frame, "load" /* Constant.LoadEvent */, generate.bind(this, frame, "childList" /* Constant.ChildList */), true);
    }
}
function stop$i() {
    for (let observer of Array.from(observers)) {
        if (observer) {
            observer.disconnect();
        }
    }
    observers = new Set();
    history$4 = {};
    mutations = [];
    throttledMutations = {};
    queue$2 = [];
    activePeriod = 0;
    timeout$1 = null;
    observedNodes = new WeakMap();
}
function active$2() {
    activePeriod = time() + 3000 /* Setting.MutationActivePeriod */;
}
function disconnect(n) {
    const ob = observedNodes.get(n);
    if (ob) {
        ob.disconnect();
        observers.delete(ob);
        observedNodes.delete(n);
    }
}
function handle$1(m) {
    handle$1.dn = 22 /* FunctionNames.MutationHandle */;
    // Queue up mutation records for asynchronous processing
    let now = time();
    track$7(6 /* Event.Mutation */, now);
    mutations.push({ time: now, mutations: m });
    schedule(process$2, 1 /* Priority.High */).then(() => {
        setTimeout(compute$6);
        measure(compute$5)();
    });
}
async function processMutation(timer, mutation, instance, timestamp) {
    let state$1 = state(timer);
    if (state$1 === 0 /* Task.Wait */) {
        state$1 = await suspend$1(timer);
    }
    if (state$1 === 2 /* Task.Stop */) {
        return;
    }
    let target = mutation.target;
    let type = config$2.throttleDom ? track$5(mutation, timer, instance, timestamp) : mutation.type;
    if (type && target && target.ownerDocument) {
        parse(target.ownerDocument);
    }
    if (type && target && target.nodeType == Node.DOCUMENT_FRAGMENT_NODE && target.host) {
        parse(target);
    }
    switch (type) {
        case "attributes" /* Constant.Attributes */:
            if (IGNORED_ATTRIBUTES.indexOf(mutation.attributeName) < 0) {
                processNode(target, 3 /* Source.Attributes */, timestamp);
            }
            break;
        case "characterData" /* Constant.CharacterData */:
            processNode(target, 4 /* Source.CharacterData */, timestamp);
            break;
        case "childList" /* Constant.ChildList */:
            processNodeList(mutation.addedNodes, 1 /* Source.ChildListAdd */, timer, timestamp);
            processNodeList(mutation.removedNodes, 2 /* Source.ChildListRemove */, timer, timestamp);
            break;
    }
}
async function process$2() {
    let timer = { id: id(), cost: 3 /* Metric.LayoutCost */ };
    start$d(timer);
    while (mutations.length > 0) {
        let record = mutations.shift();
        let instance = time();
        for (let mutation of record.mutations) {
            await processMutation(timer, mutation, instance, record.time);
        }
        await encode$4(6 /* Event.Mutation */, timer, record.time);
    }
    let processedMutations = false;
    for (var key of Object.keys(throttledMutations)) {
        let throttledMutationToProcess = throttledMutations[key];
        delete throttledMutations[key];
        await processMutation(timer, throttledMutationToProcess.mutation, time(), throttledMutationToProcess.timestamp);
        processedMutations = true;
    }
    if (Object.keys(throttledMutations).length > 0) {
        processThrottledMutations();
    }
    // ensure we encode the previously throttled mutations once we have finished them
    if (Object.keys(throttledMutations).length === 0 && processedMutations) {
        await encode$4(6 /* Event.Mutation */, timer, time());
    }
    cleanHistory();
    stop$c(timer);
}
function cleanHistory() {
    let now = time();
    if (Object.keys(history$4).length > 10000 /* Setting.MaxMutationHistoryCount */) {
        history$4 = {};
        count$1(38 /* Metric.HistoryClear */);
    }
    for (let key of Object.keys(history$4)) {
        let h = history$4[key];
        if (now > h[1] + 30000 /* Setting.MaxMutationHistoryTime */) {
            delete history$4[key];
        }
    }
}
function track$5(m, timer, instance, timestamp) {
    let value = m.target ? get(m.target.parentNode) : null;
    // Check if the parent is already discovered and that the parent is not the document root
    if (value && value.data.tag !== "HTML" /* Constant.HTML */) {
        // calculate inactive period based on the timestamp of the mutation not when the mutation is processed
        let inactive = timestamp > activePeriod;
        // Calculate critical period based on when mutation is processed
        let target = get(m.target);
        let element = target && target.selector ? target.selector.join() : m.target.nodeName;
        let parent = value.selector ? value.selector.join() : "" /* Constant.Empty */;
        // We use selector, instead of id, to determine the key (signature for the mutation) because in some cases
        // repeated mutations can cause elements to be destroyed and then recreated as new DOM nodes
        // In those cases, IDs will change however the selector (which is relative to DOM xPath) remains the same
        let key = [parent, element, m.attributeName, names(m.addedNodes), names(m.removedNodes)].join();
        // Initialize an entry if it doesn't already exist
        history$4[key] = key in history$4 ? history$4[key] : [0, instance];
        let h = history$4[key];
        // Lookup any pending nodes queued up for removal, and process them now if we suspended a mutation before
        if (inactive === false && h[0] >= 10 /* Setting.MutationSuspendThreshold */) {
            processNodeList(h[2], 2 /* Source.ChildListRemove */, timer, timestamp);
        }
        // Update the counter, do not reset counter if its critical period
        h[0] = inactive ? (h[1] === instance ? h[0] : h[0] + 1) : 1;
        h[1] = instance;
        // Return updated mutation type based on,
        // 1. if we have already hit the threshold or not
        // 2. if its a low priority mutation happening during critical time period
        if (h[0] >= 10 /* Setting.MutationSuspendThreshold */) {
            // Store a reference to removedNodes so we can process them later
            // when we resume mutations again on user interactions
            h[2] = m.removedNodes;
            if (instance > timestamp + 3000 /* Setting.MutationActivePeriod */) {
                return m.type;
            }
            // we only store the most recent mutation for a given key if it is being throttled
            throttledMutations[key] = { mutation: m, timestamp };
            return "throttle" /* Constant.Throttle */;
        }
    }
    return m.type;
}
function names(nodes) {
    let output = [];
    for (let i = 0; nodes && i < nodes.length; i++) {
        output.push(nodes[i].nodeName);
    }
    return output.join();
}
async function processNodeList(list, source, timer, timestamp) {
    let length = list ? list.length : 0;
    for (let i = 0; i < length; i++) {
        const node = list[i];
        if (source === 1 /* Source.ChildListAdd */) {
            traverse(node, timer, source, timestamp);
        }
        else {
            let state$1 = state(timer);
            if (state$1 === 0 /* Task.Wait */) {
                state$1 = await suspend$1(timer);
            }
            if (state$1 === 2 /* Task.Stop */) {
                break;
            }
            processNode(node, source, timestamp);
        }
    }
}
function processThrottledMutations() {
    if (throttleDelay) {
        clearTimeout(throttleDelay);
    }
    throttleDelay = setTimeout(() => {
        schedule(process$2, 1 /* Priority.High */);
    }, 33 /* Setting.LookAhead */);
}
function schedule$1(node) {
    // Only schedule manual trigger for this node if it's not already in the queue
    if (queue$2.indexOf(node) < 0) {
        queue$2.push(node);
    }
    // Cancel any previous trigger before scheduling a new one.
    // It's common for a webpage to call multiple synchronous "insertRule" / "deleteRule" calls.
    // And in those cases we do not wish to monitor changes multiple times for the same node.
    if (timeout$1) {
        clearTimeout(timeout$1);
    }
    timeout$1 = setTimeout(() => {
        trigger$1();
    }, 33 /* Setting.LookAhead */);
    return node;
}
function trigger$1() {
    for (let node of queue$2) {
        // Generate a mutation for this node only if it still exists
        if (node) {
            let shadowRoot = node.nodeType === Node.DOCUMENT_FRAGMENT_NODE;
            // Skip re-processing shadowRoot if it was already discovered
            if (shadowRoot && has$1(node)) {
                continue;
            }
            generate(node, shadowRoot ? "childList" /* Constant.ChildList */ : "characterData" /* Constant.CharacterData */);
        }
    }
    queue$2 = [];
}
function generate(target, type) {
    generate.dn = 23 /* FunctionNames.MutationGenerate */;
    measure(handle$1)([
        {
            addedNodes: [target],
            attributeName: null,
            attributeNamespace: null,
            nextSibling: null,
            oldValue: null,
            previousSibling: null,
            removedNodes: [],
            target,
            type,
        },
    ]);
}
function proxyStyleRules(win) {
    if (win === null || win === undefined) {
        return;
    }
    win.clarityOverrides = win.clarityOverrides || {};
    // Some popular open source libraries, like styled-components, optimize performance
    // by injecting CSS using insertRule API vs. appending text node. A side effect of
    // using javascript API is that it doesn't trigger DOM mutation and therefore we
    // need to override the insertRule API and listen for changes manually.
    if (win.clarityOverrides.InsertRule === undefined) {
        win.clarityOverrides.InsertRule = win.CSSStyleSheet.prototype.insertRule;
        win.CSSStyleSheet.prototype.insertRule = function () {
            if (active()) {
                schedule$1(this.ownerNode);
            }
            return win.clarityOverrides.InsertRule.apply(this, arguments);
        };
    }
    if ("CSSMediaRule" in win && win.clarityOverrides.MediaInsertRule === undefined) {
        win.clarityOverrides.MediaInsertRule = win.CSSMediaRule.prototype.insertRule;
        win.CSSMediaRule.prototype.insertRule = function () {
            if (active()) {
                schedule$1(this.parentStyleSheet.ownerNode);
            }
            return win.clarityOverrides.MediaInsertRule.apply(this, arguments);
        };
    }
    if (win.clarityOverrides.DeleteRule === undefined) {
        win.clarityOverrides.DeleteRule = win.CSSStyleSheet.prototype.deleteRule;
        win.CSSStyleSheet.prototype.deleteRule = function () {
            if (active()) {
                schedule$1(this.ownerNode);
            }
            return win.clarityOverrides.DeleteRule.apply(this, arguments);
        };
    }
    if ("CSSMediaRule" in win && win.clarityOverrides.MediaDeleteRule === undefined) {
        win.clarityOverrides.MediaDeleteRule = win.CSSMediaRule.prototype.deleteRule;
        win.CSSMediaRule.prototype.deleteRule = function () {
            if (active()) {
                schedule$1(this.parentStyleSheet.ownerNode);
            }
            return win.clarityOverrides.MediaDeleteRule.apply(this, arguments);
        };
    }
    // Add a hook to attachShadow API calls
    // In case we are unable to add a hook and browser throws an exception,
    // reset attachShadow variable and resume processing like before
    if (win.clarityOverrides.AttachShadow === undefined) {
        win.clarityOverrides.AttachShadow = win.Element.prototype.attachShadow;
        try {
            win.Element.prototype.attachShadow = function () {
                if (active()) {
                    return schedule$1(win.clarityOverrides.AttachShadow.apply(this, arguments));
                }
                else {
                    return win.clarityOverrides.AttachShadow.apply(this, arguments);
                }
            };
        }
        catch (_a) {
            win.clarityOverrides.AttachShadow = null;
        }
    }
}

const digitsRegex = /[^0-9\.]/g;
/* JSON+LD (Linked Data) Recursive Parser */
function ld(json) {
    for (let key of Object.keys(json)) {
        let value = json[key];
        if (key === "@type" /* JsonLD.Type */ && typeof value === "string") {
            value = value.toLowerCase();
            /* Normalizations */
            value = value.indexOf("article" /* JsonLD.Article */) >= 0 || value.indexOf("posting" /* JsonLD.Posting */) >= 0 ? "article" /* JsonLD.Article */ : value;
            switch (value) {
                case "article" /* JsonLD.Article */:
                case "recipe" /* JsonLD.Recipe */:
                    log(5 /* Dimension.SchemaType */, json[key]);
                    log(8 /* Dimension.AuthorName */, json["creator" /* JsonLD.Creator */]);
                    log(18 /* Dimension.Headline */, json["headline" /* JsonLD.Headline */]);
                    break;
                case "product" /* JsonLD.Product */:
                    log(5 /* Dimension.SchemaType */, json[key]);
                    log(10 /* Dimension.ProductName */, json["name" /* JsonLD.Name */]);
                    log(12 /* Dimension.ProductSku */, json["sku" /* JsonLD.Sku */]);
                    if (json["brand" /* JsonLD.Brand */]) {
                        log(6 /* Dimension.ProductBrand */, json["brand" /* JsonLD.Brand */]["name" /* JsonLD.Name */]);
                    }
                    break;
                case "aggregaterating" /* JsonLD.AggregateRating */:
                    if (json["ratingValue" /* JsonLD.RatingValue */]) {
                        max(11 /* Metric.RatingValue */, num$1(json["ratingValue" /* JsonLD.RatingValue */], 100 /* Setting.RatingScale */));
                        max(18 /* Metric.BestRating */, num$1(json["bestRating" /* JsonLD.BestRating */]));
                        max(19 /* Metric.WorstRating */, num$1(json["worstRating" /* JsonLD.WorstRating */]));
                    }
                    max(12 /* Metric.RatingCount */, num$1(json["ratingCount" /* JsonLD.RatingCount */]));
                    max(17 /* Metric.ReviewCount */, num$1(json["reviewCount" /* JsonLD.ReviewCount */]));
                    break;
                case "offer" /* JsonLD.Offer */:
                    log(7 /* Dimension.ProductAvailability */, json["availability" /* JsonLD.Availability */]);
                    log(14 /* Dimension.ProductCondition */, json["itemCondition" /* JsonLD.ItemCondition */]);
                    log(13 /* Dimension.ProductCurrency */, json["priceCurrency" /* JsonLD.PriceCurrency */]);
                    log(12 /* Dimension.ProductSku */, json["sku" /* JsonLD.Sku */]);
                    max(13 /* Metric.ProductPrice */, num$1(json["price" /* JsonLD.Price */]));
                    break;
                case "brand" /* JsonLD.Brand */:
                    log(6 /* Dimension.ProductBrand */, json["name" /* JsonLD.Name */]);
                    break;
            }
        }
        // Continue parsing nested objects
        if (value !== null && typeof (value) === "object" /* Constant.Object */) {
            ld(value);
        }
    }
}
function num$1(input, scale = 1) {
    if (input !== null) {
        switch (typeof input) {
            case "number" /* Constant.Number */: return Math.round(input * scale);
            case "string" /* Constant.String */: return Math.round(parseFloat(input.replace(digitsRegex, "" /* Constant.Empty */)) * scale);
        }
    }
    return null;
}

const IGNORE_ATTRIBUTES = ["title", "alt", "onload", "onfocus", "onerror", "data-drupal-form-submit-last", "aria-label"];
const newlineRegex = /[\r\n]+/g;
function processNode (node, source, timestamp) {
    var _a;
    let child = null;
    // Do not track this change if we are attempting to remove a node before discovering it
    if (source === 2 /* Source.ChildListRemove */ && has$1(node) === false) {
        return child;
    }
    // Special handling for text nodes that belong to style nodes
    if (source !== 0 /* Source.Discover */ &&
        node.nodeType === Node.TEXT_NODE &&
        node.parentElement &&
        node.parentElement.tagName === "STYLE") {
        node = node.parentNode;
    }
    let add = has$1(node) === false;
    let call = add ? "add" : "update";
    let parent = node.parentElement ? node.parentElement : null;
    let insideFrame = node.ownerDocument !== document;
    switch (node.nodeType) {
        case Node.DOCUMENT_TYPE_NODE:
            parent = insideFrame && node.parentNode ? iframe(node.parentNode) : parent;
            let docTypePrefix = insideFrame ? "iframe:" /* Constant.IFramePrefix */ : "" /* Constant.Empty */;
            let doctype = node;
            let docName = doctype.name ? doctype.name : "HTML" /* Constant.HTML */;
            let docAttributes = { name: docName, publicId: doctype.publicId, systemId: doctype.systemId };
            let docData = { tag: docTypePrefix + "*D" /* Constant.DocumentTag */, attributes: docAttributes };
            dom[call](node, parent, docData, source);
            break;
        case Node.DOCUMENT_NODE:
            // We check for regions in the beginning when discovering document and
            // later whenever there are new additions or modifications to DOM (mutations)
            if (node === document) {
                parse(document);
            }
            checkDocumentStyles(node, timestamp);
            observe$2(node);
            break;
        case Node.DOCUMENT_FRAGMENT_NODE:
            let shadowRoot = node;
            if (shadowRoot.host) {
                parse(shadowRoot);
                let type = typeof (shadowRoot.constructor);
                if (type === "function" /* Constant.Function */ && shadowRoot.constructor.toString().indexOf("[native code]" /* Constant.NativeCode */) >= 0) {
                    observe$2(shadowRoot);
                    // See: https://wicg.github.io/construct-stylesheets/ for more details on adoptedStyleSheets.
                    // At the moment, we are only able to capture "open" shadow DOM nodes. If they are closed, they are not accessible.
                    // In future we may decide to proxy "attachShadow" call to gain access, but at the moment, we don't want to
                    // cause any unintended side effect to the page. We will re-evaluate after we gather more real world data on this.
                    let style = "" /* Constant.Empty */;
                    let fragmentData = { tag: "*S" /* Constant.ShadowDomTag */, attributes: { style } };
                    dom[call](node, shadowRoot.host, fragmentData, source);
                }
                else {
                    // If the browser doesn't support shadow DOM natively, we detect that, and send appropriate tag back.
                    // The differentiation is important because we don't have to observe pollyfill shadow DOM nodes,
                    // the same way we observe real shadow DOM nodes (encapsulation provided by the browser).
                    dom[call](node, shadowRoot.host, { tag: "*P" /* Constant.PolyfillShadowDomTag */, attributes: {} }, source);
                }
                checkDocumentStyles(node, timestamp);
            }
            break;
        case Node.TEXT_NODE:
            // In IE11 TEXT_NODE doesn't expose a valid parentElement property. Instead we need to lookup parentNode property.
            parent = parent ? parent : node.parentNode;
            // Account for this text node only if we are tracking the parent node
            // We do not wish to track text nodes for ignored parent nodes, like script tags
            // Also, we do not track text nodes for STYLE tags
            // The only exception is when we receive a mutation to remove the text node, in that case
            // parent will be null, but we can still process the node by checking it's an update call.
            if (call === "update" || (parent && has$1(parent) && parent.tagName !== "STYLE" && parent.tagName !== "NOSCRIPT")) {
                let textData = { tag: "*T" /* Constant.TextTag */, value: node.nodeValue };
                dom[call](node, parent, textData, source);
            }
            break;
        case Node.ELEMENT_NODE:
            let element = node;
            let tag = element.tagName;
            let attributes = getAttributes(element);
            // In some cases, external libraries like vue-fragment, can modify parentNode property to not be in sync with the DOM
            // For correctness, we first look at parentElement and if it not present then fall back to using parentNode
            parent = node.parentElement ? node.parentElement : (node.parentNode ? node.parentNode : null);
            // If we encounter a node that is part of SVG namespace, prefix the tag with SVG_PREFIX
            if (element.namespaceURI === "http://www.w3.org/2000/svg" /* Constant.SvgNamespace */) {
                tag = "svg:" /* Constant.SvgPrefix */ + tag;
            }
            switch (tag) {
                case "HTML":
                    parent = insideFrame && parent ? iframe(parent) : parent;
                    let htmlPrefix = insideFrame ? "iframe:" /* Constant.IFramePrefix */ : "" /* Constant.Empty */;
                    let htmlData = { tag: htmlPrefix + tag, attributes };
                    dom[call](node, parent, htmlData, source);
                    break;
                case "SCRIPT":
                    if ("type" /* Constant.Type */ in attributes && attributes["type" /* Constant.Type */] === "application/ld+json" /* Constant.JsonLD */) {
                        try {
                            ld(JSON.parse(element.text.replace(newlineRegex, "" /* Constant.Empty */)));
                        }
                        catch ( /* do nothing */_b) { /* do nothing */ }
                    }
                    break;
                case "NOSCRIPT":
                    // keeping the noscript tag but ignoring its contents. Some HTML markup relies on having these tags
                    // to maintain parity with the original css view, but we don't want to execute any noscript in Clarity
                    let noscriptData = { tag, attributes: {}, value: '' };
                    dom[call](node, parent, noscriptData, source);
                    break;
                case "META":
                    var key = ("property" /* Constant.Property */ in attributes ?
                        "property" /* Constant.Property */ :
                        ("name" /* Constant.Name */ in attributes ? "name" /* Constant.Name */ : null));
                    if (key && "content" /* Constant.Content */ in attributes) {
                        let content = attributes["content" /* Constant.Content */];
                        switch (attributes[key]) {
                            case "og:title" /* Constant.ogTitle */:
                                log(20 /* Dimension.MetaTitle */, content);
                                break;
                            case "og:type" /* Constant.ogType */:
                                log(19 /* Dimension.MetaType */, content);
                                break;
                            case "generator" /* Constant.Generator */:
                                log(21 /* Dimension.Generator */, content);
                                break;
                        }
                    }
                    break;
                case "HEAD":
                    let head = { tag, attributes };
                    let l = insideFrame && ((_a = node.ownerDocument) === null || _a === void 0 ? void 0 : _a.location) ? node.ownerDocument.location : location;
                    head.attributes["*B" /* Constant.Base */] = l.protocol + "//" + l.host + l.pathname;
                    dom[call](node, parent, head, source);
                    break;
                case "BASE":
                    // Override the auto detected base path to explicit value specified in this tag
                    let baseHead = get(node.parentElement);
                    if (baseHead) {
                        // We create "a" element so we can generate protocol and hostname for relative paths like "/path/"
                        let a = document.createElement("a");
                        a.href = attributes["href"];
                        baseHead.data.attributes["*B" /* Constant.Base */] = a.protocol + "//" + a.host + a.pathname;
                    }
                    break;
                case "STYLE":
                    let styleData = { tag, attributes, value: getStyleValue(element) };
                    dom[call](node, parent, styleData, source);
                    break;
                case "IFRAME":
                    let iframe$1 = node;
                    let frameData = { tag, attributes };
                    if (sameorigin(iframe$1)) {
                        monitor(iframe$1);
                        frameData.attributes["*O" /* Constant.SameOrigin */] = "true";
                        if (iframe$1.contentDocument && iframe$1.contentWindow && iframe$1.contentDocument.readyState !== "loading") {
                            child = iframe$1.contentDocument;
                        }
                    }
                    if (source === 2 /* Source.ChildListRemove */) {
                        removeObserver(iframe$1);
                    }
                    dom[call](node, parent, frameData, source);
                    break;
                case "LINK":
                    // electron stylesheets reference the local file system - translating those
                    // to inline styles so playback can work
                    if (electron && attributes['rel'] === "stylesheet" /* Constant.StyleSheet */) {
                        for (var styleSheetIndex in Object.keys(document.styleSheets)) {
                            var currentStyleSheet = document.styleSheets[styleSheetIndex];
                            if (currentStyleSheet.ownerNode == element) {
                                let syntheticStyleData = { tag: "STYLE", attributes, value: getCssRules(currentStyleSheet) };
                                dom[call](node, parent, syntheticStyleData, source);
                                break;
                            }
                        }
                        break;
                    }
                    // for links that aren't electron style sheets we can process them normally
                    let linkData = { tag, attributes };
                    dom[call](node, parent, linkData, source);
                    break;
                case "VIDEO":
                case "AUDIO":
                case "SOURCE":
                    // Ignoring any base64 src attribute for media elements to prevent big unused tokens to be sent and shock the network
                    if ("src" /* Constant.Src */ in attributes && attributes["src" /* Constant.Src */].startsWith("data:")) {
                        attributes["src" /* Constant.Src */] = "";
                    }
                    let mediaTag = { tag, attributes };
                    dom[call](node, parent, mediaTag, source);
                    break;
                default:
                    let data = { tag, attributes };
                    if (element.shadowRoot) {
                        child = element.shadowRoot;
                    }
                    dom[call](node, parent, data, source);
                    break;
            }
            break;
    }
    return child;
}
function observe$2(root) {
    if (has$1(root) || has(root)) {
        return;
    }
    observe$3(root); // Observe mutations for this root node
    observe$4(root); // Observe interactions for this root node
}
function removeObserver(root) {
    // iframes will have load event listeners and they should be removed when iframe is removed
    // from the document
    unbind(root);
    const { doc = null, win = null } = iframeContent(root) || {};
    if (win) {
        // For iframes, scroll event is observed on content window and this needs to be removed as well
        unbind(win);
    }
    if (doc) {
        // When an iframe is removed, we should also remove all listeners attached to its document
        // to avoid memory leaks.
        unbind(doc);
        disconnect(doc);
        // Remove iframe and content document from maps tracking them
        removeIFrame(root, doc);
    }
}
function getStyleValue(style) {
    // Call trim on the text content to ensure we do not process white spaces ( , \n, \r\n, \t, etc.)
    // Also, check if stylesheet has any data-* attribute, if so process rules instead of looking up text
    // Additionally, check if style node has an id - if so it's at a high risk to have experienced dynamic
    // style updates which would make the textContent out of date with its true style contribution.
    let value = style.textContent ? style.textContent.trim() : "" /* Constant.Empty */;
    let dataset = style.dataset ? Object.keys(style.dataset).length : 0;
    if (value.length === 0 || dataset > 0 || style.id.length > 0) {
        value = getCssRules(style.sheet);
    }
    return value;
}
function getCssRules(sheet) {
    let value = "" /* Constant.Empty */;
    let cssRules = null;
    // Firefox throws a SecurityError when trying to access cssRules of a stylesheet from a different domain
    try {
        cssRules = sheet ? sheet.cssRules : [];
    }
    catch (e) {
        log$1(1 /* Code.CssRules */, 1 /* Severity.Warning */, e ? e.name : null);
        if (e && e.name !== "SecurityError") {
            throw e;
        }
    }
    if (cssRules !== null) {
        for (let i = 0; i < cssRules.length; i++) {
            value += cssRules[i].cssText;
        }
    }
    return value;
}
function getAttributes(element) {
    let output = {};
    let attributes = element.attributes;
    if (attributes && attributes.length > 0) {
        for (let i = 0; i < attributes.length; i++) {
            let name = attributes[i].name;
            if (IGNORE_ATTRIBUTES.indexOf(name) < 0) {
                output[name] = attributes[i].value;
            }
        }
    }
    // For INPUT tags read the dynamic "value" property if an explicit "value" attribute is not set
    if (element.tagName === "INPUT" /* Constant.InputTag */ && !("value" /* Constant.Value */ in output) && element.value) {
        output["value" /* Constant.Value */] = element.value;
    }
    return output;
}

const excludeClassNames = "load,active,fixed,visible,focus,show,collaps,animat" /* Constant.ExcludeClassNames */.split("," /* Constant.Comma */);
let selectorMap = {};
function reset$8() {
    selectorMap = {};
}
function get$1(input, type) {
    let a = input.attributes;
    let prefix = input.prefix ? input.prefix[type] : null;
    let suffix = type === 0 /* Selector.Alpha */ ? `${"~" /* Constant.Tilde */}${input.position - 1}` : `:nth-of-type(${input.position})`;
    switch (input.tag) {
        case "STYLE":
        case "TITLE":
        case "LINK":
        case "META":
        case "*T" /* Constant.TextTag */:
        case "*D" /* Constant.DocumentTag */:
            return "" /* Constant.Empty */;
        case "HTML":
            return "HTML" /* Constant.HTML */;
        default:
            if (prefix === null) {
                return "" /* Constant.Empty */;
            }
            prefix = `${prefix}${">" /* Constant.Separator */}`;
            input.tag = input.tag.indexOf("svg:" /* Constant.SvgPrefix */) === 0 ? input.tag.substr("svg:" /* Constant.SvgPrefix */.length) : input.tag;
            let selector = `${prefix}${input.tag}${suffix}`;
            let id = "id" /* Constant.Id */ in a && a["id" /* Constant.Id */].length > 0 ? a["id" /* Constant.Id */] : null;
            let classes = input.tag !== "BODY" /* Constant.BodyTag */ && "class" /* Constant.Class */ in a && a["class" /* Constant.Class */].length > 0 ? a["class" /* Constant.Class */].trim().split(/\s+/).filter(c => filter(c)).join("." /* Constant.Period */) : null;
            if (classes && classes.length > 0) {
                if (type === 0 /* Selector.Alpha */) {
                    // In Alpha mode, update selector to use class names, with relative positioning within the parent id container.
                    // If the node has valid class name(s) then drop relative positioning within the parent path to keep things simple.
                    let key = `${getDomPath(prefix)}${input.tag}${"." /* Constant.Dot */}${classes}`;
                    if (!(key in selectorMap)) {
                        selectorMap[key] = [];
                    }
                    if (selectorMap[key].indexOf(input.id) < 0) {
                        selectorMap[key].push(input.id);
                    }
                    selector = `${key}${"~" /* Constant.Tilde */}${selectorMap[key].indexOf(input.id)}`;
                }
                else {
                    // In Beta mode, we continue to look at query selectors in context of the full page
                    selector = `${prefix}${input.tag}.${classes}${suffix}`;
                }
            }
            // Update selector to use "id" field when available. There are two exceptions:
            // (1) if "id" appears to be an auto generated string token, e.g. guid or a random id containing digits
            // (2) if "id" appears inside a shadow DOM, in which case we continue to prefix up to shadow DOM to prevent conflicts
            selector = id && filter(id) ? `${getDomPrefix(prefix)}${"#" /* Constant.Hash */}${id}` : selector;
            return selector;
    }
}
function getDomPrefix(prefix) {
    const shadowDomStart = prefix.lastIndexOf("*S" /* Constant.ShadowDomTag */);
    const iframeDomStart = prefix.lastIndexOf(`${"iframe:" /* Constant.IFramePrefix */}${"HTML" /* Constant.HTML */}`);
    const domStart = Math.max(shadowDomStart, iframeDomStart);
    if (domStart < 0) {
        return "" /* Constant.Empty */;
    }
    return prefix.substring(0, prefix.indexOf(">" /* Constant.Separator */, domStart) + 1);
}
function getDomPath(input) {
    let parts = input.split(">" /* Constant.Separator */);
    for (let i = 0; i < parts.length; i++) {
        let tIndex = parts[i].indexOf("~" /* Constant.Tilde */);
        let dIndex = parts[i].indexOf("." /* Constant.Dot */);
        parts[i] = parts[i].substring(0, dIndex > 0 ? dIndex : (tIndex > 0 ? tIndex : parts[i].length));
    }
    return parts.join(">" /* Constant.Separator */);
}
// Check if the given input string has digits or excluded class names
function filter(value) {
    if (!value) {
        return false;
    } // Do not process empty strings
    if (excludeClassNames.some(x => value.toLowerCase().indexOf(x) >= 0)) {
        return false;
    }
    for (let i = 0; i < value.length; i++) {
        let c = value.charCodeAt(i);
        if (c >= 48 /* Character.Zero */ && c <= 57 /* Character.Nine */) {
            return false;
        }
    }
    return true;
}

var selector = /*#__PURE__*/Object.freeze({
    __proto__: null,
    get: get$1,
    reset: reset$8
});

let index = 1;
let nodesMap = null; // Maps id => node to retrieve further node details using id.
let values = [];
let updateMap = [];
let hashMap = {};
let override = [];
let unmask = [];
let maskText = [];
let maskExclude = [];
let maskDisable = [];
let maskTags = [];
// The WeakMap object is a collection of key/value pairs in which the keys are weakly referenced
let idMap = null; // Maps node => id.
let iframeMap = null; // Maps iframe's contentDocument => parent iframe element
let iframeContentMap = null; // Maps parent iframe element => iframe's contentDocument & contentWindow
let privacyMap = null; // Maps node => Privacy (enum)
let fraudMap = null; // Maps node => FraudId (number)
function start$j() {
    reset$7();
    parse(document, true);
}
function stop$h() {
    reset$7();
}
function reset$7() {
    index = 1;
    values = [];
    updateMap = [];
    hashMap = {};
    override = [];
    unmask = [];
    maskText = "address,password,contact" /* Mask.Text */.split("," /* Constant.Comma */);
    maskExclude = "password,secret,pass,social,ssn,code,hidden" /* Mask.Exclude */.split("," /* Constant.Comma */);
    maskDisable = "radio,checkbox,range,button,reset,submit" /* Mask.Disable */.split("," /* Constant.Comma */);
    maskTags = "INPUT,SELECT,TEXTAREA" /* Mask.Tags */.split("," /* Constant.Comma */);
    nodesMap = new Map();
    idMap = new WeakMap();
    iframeMap = new WeakMap();
    iframeContentMap = new WeakMap();
    privacyMap = new WeakMap();
    fraudMap = new WeakMap();
    reset$8();
}
// We parse new root nodes for any regions or masked nodes in the beginning (document) and
// later whenever there are new additions or modifications to DOM (mutations)
function parse(root, init = false) {
    // Wrap selectors in a try / catch block.
    // It's possible for script to receive invalid selectors, e.g. "'#id'" with extra quotes, and cause the code below to fail
    try {
        // Parse unmask configuration into separate query selectors and override tokens as part of initialization
        if (init) {
            config$2.unmask.forEach(x => x.indexOf("!" /* Constant.Bang */) < 0 ? unmask.push(x) : override.push(x.substr(1)));
        }
        // Since mutations may happen on leaf nodes too, e.g. text nodes, which may not support all selector APIs.
        // We ensure that the root note supports querySelectorAll API before executing the code below to identify new regions.
        if ("querySelectorAll" in root) {
            config$2.regions.forEach(x => root.querySelectorAll(x[1]).forEach(e => observe$1(e, `${x[0]}`))); // Regions
            config$2.mask.forEach(x => root.querySelectorAll(x).forEach(e => privacyMap.set(e, 3 /* Privacy.TextImage */))); // Masked Elements
            config$2.checksum.forEach(x => root.querySelectorAll(x[1]).forEach(e => fraudMap.set(e, x[0]))); // Fraud Checksum Check
            unmask.forEach(x => root.querySelectorAll(x).forEach(e => privacyMap.set(e, 0 /* Privacy.None */))); // Unmasked Elements
        }
    }
    catch (e) {
        log$1(5 /* Code.Selector */, 1 /* Severity.Warning */, e ? e.name : null);
    }
}
function getId(node, autogen = false) {
    if (node === null) {
        return null;
    }
    let id = idMap.get(node);
    if (!id && autogen) {
        id = index++;
        idMap.set(node, id);
    }
    return id ? id : null;
}
function add(node, parent, data, source) {
    let parentId = parent ? getId(parent) : null;
    // Do not add detached nodes
    if ((!parent || !parentId) && node.host == null && node.nodeType !== Node.DOCUMENT_TYPE_NODE) {
        return;
    }
    let id = getId(node, true);
    let previousId = getPreviousId(node);
    let parentValue = null;
    let regionId = exists(node) ? id : null;
    let fraudId = fraudMap.has(node) ? fraudMap.get(node) : null;
    let privacyId = config$2.content ? 1 /* Privacy.Sensitive */ : 3 /* Privacy.TextImage */;
    if (parentId >= 0 && values[parentId]) {
        parentValue = values[parentId];
        parentValue.children.push(id);
        regionId = regionId === null ? parentValue.region : regionId;
        fraudId = fraudId === null ? parentValue.metadata.fraud : fraudId;
        privacyId = parentValue.metadata.privacy;
    }
    // If there's an explicit region attribute set on the element, use it to mark a region on the page
    if (data.attributes && "data-clarity-region" /* Constant.RegionData */ in data.attributes) {
        observe$1(node, data.attributes["data-clarity-region" /* Constant.RegionData */]);
        regionId = id;
    }
    nodesMap.set(id, node);
    values[id] = {
        id,
        parent: parentId,
        previous: previousId,
        children: [],
        data,
        selector: null,
        hash: null,
        region: regionId,
        metadata: { active: true, suspend: false, privacy: privacyId, position: null, fraud: fraudId, size: null },
    };
    privacy(node, values[id], parentValue);
    updateSelector(values[id]);
    updateImageSize(values[id]);
    track$4(id, source);
}
function update(node, parent, data, source) {
    let id = getId(node);
    let parentId = parent ? getId(parent) : null;
    let previousId = getPreviousId(node);
    let changed = false;
    let parentChanged = false;
    if (id in values) {
        let value = values[id];
        value.metadata.active = true;
        // Handle case where internal ordering may have changed
        if (value.previous !== previousId) {
            changed = true;
            value.previous = previousId;
        }
        // Handle case where parent might have been updated
        if (value.parent !== parentId) {
            changed = true;
            let oldParentId = value.parent;
            value.parent = parentId;
            // Move this node to the right location under new parent
            if (parentId !== null && parentId >= 0) {
                let childIndex = previousId === null ? 0 : values[parentId].children.indexOf(previousId) + 1;
                values[parentId].children.splice(childIndex, 0, id);
                // Update region after the move
                value.region = exists(node) ? id : values[parentId].region;
            }
            else {
                // Mark this element as deleted if the parent has been updated to null
                remove(id, source);
            }
            // Remove reference to this node from the old parent
            if (oldParentId !== null && oldParentId >= 0) {
                let nodeIndex = values[oldParentId].children.indexOf(id);
                if (nodeIndex >= 0) {
                    values[oldParentId].children.splice(nodeIndex, 1);
                }
            }
            parentChanged = true;
        }
        // Update data
        for (let key in data) {
            if (diff(value["data"], data, key)) {
                changed = true;
                value["data"][key] = data[key];
            }
        }
        // Update selector
        updateSelector(value);
        track$4(id, source, changed, parentChanged);
    }
}
function sameorigin(node) {
    let output = false;
    if (node.nodeType === Node.ELEMENT_NODE && node.tagName === "IFRAME" /* Constant.IFrameTag */) {
        let frame = node;
        // To determine if the iframe is same-origin or not, we try accessing it's contentDocument.
        // If the browser throws an exception, we assume it's cross-origin and move on.
        // However, if we do a get a valid document object back, we assume the contents are accessible and iframe is same-origin.
        try {
            let doc = frame.contentDocument;
            if (doc) {
                iframeMap.set(frame.contentDocument, frame);
                iframeContentMap.set(frame, { doc: frame.contentDocument, win: frame.contentWindow });
                output = true;
            }
        }
        catch ( /* do nothing */_a) { /* do nothing */ }
    }
    return output;
}
function iframe(node) {
    let doc = node.nodeType === Node.DOCUMENT_NODE ? node : null;
    return doc && iframeMap.has(doc) ? iframeMap.get(doc) : null;
}
function iframeContent(frame) {
    if (iframeContentMap.has(frame)) {
        return iframeContentMap.get(frame);
    }
    return null;
}
function removeIFrame(frame, doc) {
    iframeContentMap.delete(frame);
    iframeMap.delete(doc);
}
function privacy(node, value, parent) {
    var _a;
    let data = value.data;
    let metadata = value.metadata;
    let current = metadata.privacy;
    let attributes = data.attributes || {};
    let tag = data.tag.toUpperCase();
    switch (true) {
        case maskTags.indexOf(tag) >= 0:
            let type = attributes["type" /* Constant.Type */];
            let meta = "" /* Constant.Empty */;
            const excludedPrivacyAttributes = ["class" /* Constant.Class */, "style" /* Constant.Style */];
            Object.keys(attributes)
                .filter((x) => !excludedPrivacyAttributes.includes(x))
                .forEach((x) => (meta += attributes[x].toLowerCase()));
            let exclude = maskExclude.some((x) => meta.indexOf(x) >= 0);
            // Regardless of privacy mode, always mask off user input from input boxes or drop downs with two exceptions:
            // (1) The node is detected to be one of the excluded fields, in which case we drop everything
            // (2) The node's type is one of the allowed types (like checkboxes)
            metadata.privacy = tag === "INPUT" /* Constant.InputTag */ && maskDisable.indexOf(type) >= 0 ? current : (exclude ? 4 /* Privacy.Exclude */ : 2 /* Privacy.Text */);
            break;
        case "data-clarity-mask" /* Constant.MaskData */ in attributes:
            metadata.privacy = 3 /* Privacy.TextImage */;
            break;
        case "data-clarity-unmask" /* Constant.UnmaskData */ in attributes:
            metadata.privacy = 0 /* Privacy.None */;
            break;
        case privacyMap.has(node):
            // If this node was explicitly configured to contain sensitive content, honor that privacy setting
            metadata.privacy = privacyMap.get(node);
            break;
        case fraudMap.has(node):
            // If this node was explicitly configured to be evaluated for fraud, then also mask content
            metadata.privacy = 2 /* Privacy.Text */;
            break;
        case tag === "*T" /* Constant.TextTag */:
            // If it's a text node belonging to a STYLE or TITLE tag or one of scrub exceptions, then capture content
            let pTag = parent && parent.data ? parent.data.tag : "" /* Constant.Empty */;
            let pSelector = parent && parent.selector ? parent.selector[1 /* Selector.Default */] : "" /* Constant.Empty */;
            let tags = ["STYLE" /* Constant.StyleTag */, "TITLE" /* Constant.TitleTag */, "svg:style" /* Constant.SvgStyle */];
            metadata.privacy = tags.includes(pTag) || override.some(x => pSelector.indexOf(x) >= 0) ? 0 /* Privacy.None */ : current;
            break;
        case current === 1 /* Privacy.Sensitive */:
            // In a mode where we mask sensitive information by default, look through class names to aggressively mask content
            metadata.privacy = inspect(attributes["class" /* Constant.Class */], maskText, metadata);
            break;
        case tag === "IMG" /* Constant.ImageTag */:
            // Mask images with blob src as it is not publicly available anyway.
            if ((_a = attributes.src) === null || _a === void 0 ? void 0 : _a.startsWith('blob:')) {
                metadata.privacy = 3 /* Privacy.TextImage */;
            }
            break;
    }
}
function inspect(input, lookup, metadata) {
    if (input && lookup.some(x => input.indexOf(x) >= 0)) {
        return 2 /* Privacy.Text */;
    }
    return metadata.privacy;
}
function diff(a, b, field) {
    if (typeof a[field] === "object" && typeof b[field] === "object") {
        for (let key in a[field]) {
            if (a[field][key] !== b[field][key]) {
                return true;
            }
        }
        for (let key in b[field]) {
            if (b[field][key] !== a[field][key]) {
                return true;
            }
        }
        return false;
    }
    return a[field] !== b[field];
}
function position(parent, child) {
    child.metadata.position = 1;
    let idx = parent ? parent.children.indexOf(child.id) : -1;
    while (idx-- > 0) {
        let sibling = values[parent.children[idx]];
        if (child.data.tag === sibling.data.tag) {
            child.metadata.position = sibling.metadata.position + 1;
            break;
        }
    }
    return child.metadata.position;
}
function updateSelector(value) {
    let parent = value.parent && value.parent in values ? values[value.parent] : null;
    let prefix = parent ? parent.selector : null;
    let d = value.data;
    let p = position(parent, value);
    let s = { id: value.id, tag: d.tag, prefix, position: p, attributes: d.attributes };
    value.selector = [get$1(s, 0 /* Selector.Alpha */), get$1(s, 1 /* Selector.Beta */)];
    value.hash = value.selector.map(x => x ? hash(x) : null);
    value.hash.forEach(h => hashMap[h] = value.id);
}
function hashText(hash) {
    let id = lookup(hash);
    let node = getNode(id);
    return node !== null && node.textContent !== null ? node.textContent.substr(0, 25 /* Setting.ClickText */) : '';
}
function getNode(id) {
    return nodesMap.has(id) ? nodesMap.get(id) : null;
}
function getValue(id) {
    if (id in values) {
        return values[id];
    }
    return null;
}
function get(node) {
    let id = getId(node);
    return id in values ? values[id] : null;
}
function lookup(hash) {
    return hash in hashMap ? hashMap[hash] : null;
}
function has$1(node) {
    return nodesMap.has(getId(node));
}
function updates$2() {
    let output = [];
    for (let id of updateMap) {
        if (id in values) {
            output.push(values[id]);
        }
    }
    updateMap = [];
    return output;
}
function remove(id, source) {
    if (id in values) {
        let value = values[id];
        value.metadata.active = false;
        value.parent = null;
        track$4(id, source);
        // Clean up node references for removed nodes
        removeNodeFromNodesMap(id);
    }
}
function removeNodeFromNodesMap(id) {
    const nodeToBeRemoved = nodesMap.get(id);
    // Shadow dom roots shouldn't be deleted, 
    // we should keep listening to the mutations there even they're not rendered in the DOM.
    if ((nodeToBeRemoved === null || nodeToBeRemoved === void 0 ? void 0 : nodeToBeRemoved.nodeType) === Node.DOCUMENT_FRAGMENT_NODE) {
        return;
    }
    if (nodeToBeRemoved && (nodeToBeRemoved === null || nodeToBeRemoved === void 0 ? void 0 : nodeToBeRemoved.nodeType) === Node.ELEMENT_NODE && nodeToBeRemoved["tagName"] === "IFRAME") {
        const iframe = nodeToBeRemoved;
        removeObserver(iframe);
    }
    nodesMap.delete(id);
    let value = id in values ? values[id] : null;
    if (value && value.children) {
        for (let childId of value.children) {
            removeNodeFromNodesMap(childId);
        }
    }
}
function updateImageSize(value) {
    // If this element is a image node, and is masked, then track box model for the current element
    if (value.data.tag === "IMG" /* Constant.ImageTag */ && value.metadata.privacy === 3 /* Privacy.TextImage */) {
        let img = getNode(value.id);
        // We will not capture the natural image dimensions until it loads. 
        if (img && (!img.complete || img.naturalWidth === 0)) {
            // This will trigger mutation to update the original width and height after image loads.
            bind(img, 'load', () => {
                img.setAttribute('data-clarity-loaded', `${shortid()}`);
            });
        }
        value.metadata.size = [];
    }
}
function getPreviousId(node) {
    let id = null;
    // Some nodes may not have an ID by design since Clarity skips over tags like SCRIPT, NOSCRIPT, META, COMMENTS, etc..
    // In that case, we keep going back and check for their sibling until we find a sibling with ID or no more sibling nodes are left.
    while (id === null && node.previousSibling) {
        id = getId(node.previousSibling);
        node = node.previousSibling;
    }
    return id;
}
function track$4(id, source, changed = true, parentChanged = false) {
    if (config$2.lean && config$2.lite) {
        return;
    }
    // Keep track of the order in which mutations happened, they may not be sequential
    // Edge case: If an element is added later on, and pre-discovered element is moved as a child.
    // In that case, we need to reorder the pre-discovered element in the update list to keep visualization consistent.
    let uIndex = updateMap.indexOf(id);
    if (uIndex >= 0 && source === 1 /* Source.ChildListAdd */ && parentChanged) {
        updateMap.splice(uIndex, 1);
        updateMap.push(id);
    }
    else if (uIndex === -1 && changed) {
        updateMap.push(id);
    }
}

let state$2 = [];
let regionMap = null; // Maps region nodes => region name
let regions = {};
let queue$1 = [];
let watch = false;
let observer$1 = null;
function start$i() {
    reset$6();
    observer$1 = null;
    regionMap = new WeakMap();
    regions = {};
    queue$1 = [];
    watch = window["IntersectionObserver"] ? true : false;
}
function observe$1(node, name) {
    if (regionMap.has(node) === false) {
        regionMap.set(node, name);
        observer$1 = observer$1 === null && watch ? new IntersectionObserver(handler$1, {
            // Get notified as intersection continues to change
            // This allows us to process regions that get partially hidden during the lifetime of the page
            // See: https://developer.mozilla.org/en-US/docs/Web/API/Intersection_Observer_API#creating_an_intersection_observer
            // By default, intersection observers only fire an event when even a single pixel is visible and not thereafter.
            threshold: [0, 0.05, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1]
        }) : observer$1;
        if (observer$1 && node && node.nodeType === Node.ELEMENT_NODE) {
            observer$1.observe(node);
        }
    }
}
function exists(node) {
    // Check if regionMap is not null before looking up a node
    // Since, dom module stops after region module, it's possible that we may set regionMap to be null
    // and still attempt to call exists on a late coming DOM mutation (or addition), effectively causing a script error
    return regionMap && regionMap.has(node);
}
function track$3(id, event) {
    let node = getNode(id);
    let data = id in regions ? regions[id] : { id, visibility: 0 /* RegionVisibility.Rendered */, interaction: 16 /* InteractionState.None */, name: regionMap.get(node) };
    // Determine the interaction state based on incoming event
    let interaction = 16 /* InteractionState.None */;
    switch (event) {
        case 9 /* Event.Click */:
            interaction = 20 /* InteractionState.Clicked */;
            break;
        case 27 /* Event.Input */:
            interaction = 30 /* InteractionState.Input */;
            break;
    }
    // Process updates to this region, if applicable
    process$1(node, data, interaction, data.visibility);
}
function compute$5() {
    compute$5.dn = 24 /* FunctionNames.RegionCompute */;
    // Process any regions where we couldn't resolve an "id" for at the time of last intersection observer event
    // This could happen in cases where elements are not yet processed by Clarity's virtual DOM but browser reports a change, regardless.
    // For those cases we add them to the queue and re-process them below
    let q = [];
    for (let r of queue$1) {
        let id = getId(r.node);
        if (id) {
            r.state.data.id = id;
            regions[id] = r.state.data;
            state$2.push(r.state);
        }
        else {
            q.push(r);
        }
    }
    queue$1 = q;
    // Schedule encode only when we have at least one valid data entry
    if (state$2.length > 0) {
        encode$4(7 /* Event.Region */);
    }
}
function handler$1(entries) {
    for (let entry of entries) {
        let target = entry.target;
        let rect = entry.boundingClientRect;
        let overlap = entry.intersectionRect;
        let viewport = entry.rootBounds;
        // Only capture regions that have non-zero width or height to avoid tracking and sending regions
        // that cannot ever be seen by the user. In some cases, websites will have a multiple copy of the same region
        // like search box - one for desktop, and another for mobile. In those cases, CSS media queries determine which one should be visible.
        // Also, if these regions ever become non-zero width or height (through AJAX, user action or orientation change) - we will automatically start monitoring them from that point onwards
        if (regionMap.has(target) && rect.width + rect.height > 0 && viewport.width > 0 && viewport.height > 0) {
            let id = target ? getId(target) : null;
            let data = id in regions ? regions[id] : { id, name: regionMap.get(target), interaction: 16 /* InteractionState.None */, visibility: 0 /* RegionVisibility.Rendered */ };
            // For regions that have relatively smaller area, we look at intersection ratio and see the overlap relative to element's area
            // However, for larger regions, area of regions could be bigger than viewport and therefore comparison is relative to visible area
            let viewportRatio = overlap ? (overlap.width * overlap.height * 1.0) / (viewport.width * viewport.height) : 0;
            let visible = viewportRatio > 0.05 /* Setting.ViewportIntersectionRatio */ || entry.intersectionRatio > 0.8 /* Setting.IntersectionRatio */;
            // If an element is either visible or was visible and has been scrolled to the end
            // i.e. Scrolled to end is determined by if the starting position of the element + the window height is more than the total element height. 
            // starting position is relative to the viewport - so Intersection observer returns a negative value for rect.top to indicate that the element top is above the viewport
            let scrolledToEnd = (visible || data.visibility == 10 /* RegionVisibility.Visible */) && Math.abs(rect.top) + viewport.height > rect.height;
            // Process updates to this region, if applicable
            process$1(target, data, data.interaction, (scrolledToEnd ?
                13 /* RegionVisibility.ScrolledToEnd */ :
                (visible ? 10 /* RegionVisibility.Visible */ : 0 /* RegionVisibility.Rendered */)));
            // Stop observing this element now that we have already received scrolled signal
            if (data.visibility >= 13 /* RegionVisibility.ScrolledToEnd */ && observer$1) {
                observer$1.unobserve(target);
            }
        }
    }
    if (state$2.length > 0) {
        encode$4(7 /* Event.Region */);
    }
}
function process$1(n, d, s, v) {
    // Check if received a state that supersedes existing state
    let updated = s > d.interaction || v > d.visibility;
    d.interaction = s > d.interaction ? s : d.interaction;
    d.visibility = v > d.visibility ? v : d.visibility;
    // If the corresponding node is already discovered, update the internal state
    // Otherwise, track it in a queue to reprocess later.
    if (d.id) {
        if ((d.id in regions && updated) || !(d.id in regions)) {
            regions[d.id] = d;
            state$2.push(clone(d));
        }
    }
    else {
        // Get the time before adding to queue to ensure accurate event time
        queue$1.push({ node: n, state: clone(d) });
    }
}
function clone(r) {
    return { time: time(), data: { id: r.id, interaction: r.interaction, visibility: r.visibility, name: r.name } };
}
function reset$6() {
    state$2 = [];
}
function stop$g() {
    reset$6();
    regionMap = null;
    regions = {};
    queue$1 = [];
    if (observer$1) {
        observer$1.disconnect();
        observer$1 = null;
    }
    watch = false;
}

function target(evt) {
    let path = evt.composed && evt.composedPath ? evt.composedPath() : null;
    let node = (path && path.length > 0 ? path[0] : evt.target);
    active$2(); // Mark active periods of time so mutations can continue uninterrupted
    return node && node.nodeType === Node.DOCUMENT_NODE ? node.documentElement : node;
}
function metadata$2(node, event, text = null) {
    // If the node is null, we return a reserved value for id: 0. Valid assignment of id begins from 1+.
    let output = { id: 0, hash: null, privacy: 2 /* Privacy.Text */ };
    if (node) {
        let value = get(node);
        if (value !== null) {
            let metadata = value.metadata;
            output.id = value.id;
            output.hash = value.hash;
            output.privacy = metadata.privacy;
            if (value.region) {
                track$3(value.region, event);
            }
            if (metadata.fraud) {
                check$4(metadata.fraud, value.id, text || value.data.value);
            }
        }
    }
    return output;
}

async function encode$3 (type, ts = null) {
    let t = ts || time();
    let tokens = [t, type];
    switch (type) {
        case 13 /* Event.MouseDown */:
        case 14 /* Event.MouseUp */:
        case 12 /* Event.MouseMove */:
        case 15 /* Event.MouseWheel */:
        case 16 /* Event.DoubleClick */:
        case 17 /* Event.TouchStart */:
        case 18 /* Event.TouchEnd */:
        case 19 /* Event.TouchMove */:
        case 20 /* Event.TouchCancel */:
            for (let entry of state$6) {
                let pTarget = metadata$2(entry.data.target, entry.event);
                if (pTarget.id > 0) {
                    tokens = [entry.time, entry.event];
                    tokens.push(pTarget.id);
                    tokens.push(entry.data.x);
                    tokens.push(entry.data.y);
                    if (entry.data.id !== undefined) {
                        tokens.push(entry.data.id);
                        if (entry.data.isPrimary !== undefined) {
                            tokens.push(entry.data.isPrimary.toString());
                        }
                    }
                    queue(tokens);
                    track$8(entry.event, entry.data.x, entry.data.y, entry.time);
                }
            }
            reset$i();
            break;
        case 9 /* Event.Click */:
            for (let entry of state$9) {
                let cTarget = metadata$2(entry.data.target, entry.event, entry.data.text);
                tokens = [entry.time, entry.event];
                let cHash = cTarget.hash ? cTarget.hash.join("." /* Constant.Dot */) : "" /* Constant.Empty */;
                tokens.push(cTarget.id);
                tokens.push(entry.data.x);
                tokens.push(entry.data.y);
                tokens.push(entry.data.eX);
                tokens.push(entry.data.eY);
                tokens.push(entry.data.button);
                tokens.push(entry.data.reaction);
                tokens.push(entry.data.context);
                tokens.push(text$1(entry.data.text, "click", cTarget.privacy));
                tokens.push(url$1(entry.data.link));
                tokens.push(cHash);
                tokens.push(entry.data.trust);
                tokens.push(entry.data.isFullText);
                queue(tokens);
                track$2(entry.time, entry.event, cHash, entry.data.x, entry.data.y, entry.data.reaction, entry.data.context);
            }
            reset$l();
            break;
        case 38 /* Event.Clipboard */:
            for (let entry of state$8) {
                tokens = [entry.time, entry.event];
                let target = metadata$2(entry.data.target, entry.event);
                if (target.id > 0) {
                    tokens.push(target.id);
                    tokens.push(entry.data.action);
                    queue(tokens);
                }
            }
            reset$k();
            break;
        case 11 /* Event.Resize */:
            let r = data$c;
            tokens.push(r.width);
            tokens.push(r.height);
            track$8(type, r.width, r.height);
            reset$h();
            queue(tokens);
            break;
        case 26 /* Event.Unload */:
            let u = data$a;
            tokens.push(u.name);
            tokens.push(u.persisted);
            reset$d();
            queue(tokens);
            break;
        case 27 /* Event.Input */:
            for (let entry of state$7) {
                let iTarget = metadata$2(entry.data.target, entry.event, entry.data.value);
                tokens = [entry.time, entry.event];
                tokens.push(iTarget.id);
                tokens.push(text$1(entry.data.value, "input", iTarget.privacy, false, entry.data.type));
                queue(tokens);
            }
            reset$j();
            break;
        case 21 /* Event.Selection */:
            let s = data$b;
            if (s) {
                let startTarget = metadata$2(s.start, type);
                let endTarget = metadata$2(s.end, type);
                tokens.push(startTarget.id);
                tokens.push(s.startOffset);
                tokens.push(endTarget.id);
                tokens.push(s.endOffset);
                reset$f();
                queue(tokens);
            }
            break;
        case 10 /* Event.Scroll */:
            for (let entry of state$5) {
                let sTarget = metadata$2(entry.data.target, entry.event);
                const top = metadata$2(entry.data.top, entry.event);
                const bottom = metadata$2(entry.data.bottom, entry.event);
                const sTopHash = (top === null || top === void 0 ? void 0 : top.hash) ? top.hash.join("." /* Constant.Dot */) : "" /* Constant.Empty */;
                const sBottomHash = (bottom === null || bottom === void 0 ? void 0 : bottom.hash) ? bottom.hash.join("." /* Constant.Dot */) : "" /* Constant.Empty */;
                if (sTarget.id > 0) {
                    tokens = [entry.time, entry.event];
                    tokens.push(sTarget.id);
                    tokens.push(entry.data.x);
                    tokens.push(entry.data.y);
                    tokens.push(sTopHash);
                    tokens.push(sBottomHash);
                    queue(tokens);
                    track$8(entry.event, entry.data.x, entry.data.y, entry.time);
                }
            }
            reset$g();
            break;
        case 42 /* Event.Change */:
            for (let entry of state$a) {
                tokens = [entry.time, entry.event];
                let target = metadata$2(entry.data.target, entry.event);
                if (target.id > 0) {
                    tokens = [entry.time, entry.event];
                    tokens.push(target.id);
                    tokens.push(entry.data.type);
                    tokens.push(text$1(entry.data.value, "change", target.privacy));
                    tokens.push(text$1(entry.data.checksum, "checksum", target.privacy));
                    queue(tokens);
                }
            }
            reset$m();
            break;
        case 39 /* Event.Submit */:
            for (let entry of state$4) {
                tokens = [entry.time, entry.event];
                let target = metadata$2(entry.data.target, entry.event);
                if (target.id > 0) {
                    tokens.push(target.id);
                    queue(tokens);
                }
            }
            reset$e();
            break;
        case 22 /* Event.Timeline */:
            for (let entry of updates$1) {
                tokens = [entry.time, entry.event];
                tokens.push(entry.data.type);
                tokens.push(entry.data.hash);
                tokens.push(entry.data.x);
                tokens.push(entry.data.y);
                tokens.push(entry.data.reaction);
                tokens.push(entry.data.context);
                queue(tokens, false);
            }
            reset$5();
            break;
        case 28 /* Event.Visibility */:
            let v = data$9;
            tokens.push(v.visible);
            queue(tokens);
            visibility(t, v.visible);
            reset$c();
            break;
    }
}

let state$1 = [];
let updates$1 = [];
function start$h() {
    state$1 = [];
    reset$5();
}
function reset$5() {
    updates$1 = [];
}
function track$2(time, event, hash, x, y, reaction = 1 /* BooleanFlag.True */, context = 0 /* BrowsingContext.Self */) {
    state$1.push({
        time,
        event: 22 /* Event.Timeline */,
        data: {
            type: event,
            hash,
            x,
            y,
            reaction,
            context
        }
    });
    // Since timeline only keeps the data for configured time, we still want to continue tracking these values
    // as part of the baseline. For instance, in a scenario where last scroll happened 5s ago.
    // We would still need to capture the last scroll position as part of the baseline event, even when timeline will be empty.
    track$8(event, x, y, time);
}
function compute$4() {
    const temp = [];
    updates$1 = [];
    let max = data$1.start + data$1.duration;
    let min = Math.max(max - 2000 /* Setting.TimelineSpan */, 0);
    for (let s of state$1) {
        if (s.time >= min) {
            if (s.time <= max) {
                updates$1.push(s);
            }
            temp.push(s);
        }
    }
    state$1 = temp; // Drop events less than the min time
    encode$3(22 /* Event.Timeline */);
}
function stop$f() {
    state$1 = [];
    reset$5();
}

let discoverBytes = 0;
let playbackBytes = 0;
let playback;
let analysis;
let timeout = null;
let transit;
let active$1;
let queuedTime = 0;
let track$1;
function start$g() {
    active$1 = true;
    discoverBytes = 0;
    playbackBytes = 0;
    queuedTime = 0;
    playback = [];
    analysis = [];
    transit = {};
    track$1 = null;
}
function queue(tokens, transmit = true) {
    if (active$1) {
        let now = time();
        let type = tokens.length > 1 ? tokens[1] : null;
        let event = JSON.stringify(tokens);
        switch (type) {
            case 5 /* Event.Discover */:
                discoverBytes += event.length;
            case 37 /* Event.Box */:
            case 6 /* Event.Mutation */:
            case 43 /* Event.Snapshot */:
            case 45 /* Event.StyleSheetAdoption */:
            case 46 /* Event.StyleSheetUpdate */:
                playbackBytes += event.length;
                playback.push(event);
                break;
            default:
                analysis.push(event);
                break;
        }
        // Increment event count metric
        count$1(25 /* Metric.EventCount */);
        // Following two checks are precautionary and act as a fail safe mechanism to get out of unexpected situations.
        // Check 1: If for any reason the upload hasn't happened after waiting for 2x the config.delay time,
        // reset the timer. This allows Clarity to attempt an upload again.
        let gap = delay();
        if (now - queuedTime > (gap * 2)) {
            clearTimeout(timeout);
            timeout = null;
        }
        // Transmit Check: When transmit is set to true (default), it indicates that we should schedule an upload
        // However, in certain scenarios - like metric calculation - which are triggered as part of an existing upload
        // We enrich the data going out with the existing upload. In these cases, call to upload comes with 'transmit' set to false.
        if (transmit && timeout === null) {
            if (type !== 25 /* Event.Ping */) {
                reset$q();
            }
            timeout = setTimeout(upload, gap);
            queuedTime = now;
            check$2(playbackBytes);
        }
    }
}
function stop$e() {
    clearTimeout(timeout);
    upload(true);
    discoverBytes = 0;
    playbackBytes = 0;
    queuedTime = 0;
    playback = [];
    analysis = [];
    transit = {};
    track$1 = null;
    active$1 = false;
}
async function upload(final = false) {
    timeout = null;
    // Check if we can send playback bytes over the wire or not
    // For better instrumentation coverage, we send playback bytes from second sequence onwards
    // And, we only send playback metric when we are able to send the playback bytes back to server
    let sendPlaybackBytes = config$2.lean === false && playbackBytes > 0 && (playbackBytes < 1048576 /* Setting.MaxFirstPayloadBytes */ || data$1.sequence > 0);
    if (sendPlaybackBytes) {
        max(1 /* Metric.Playback */, 1 /* BooleanFlag.True */);
    }
    // CAUTION: Ensure "transmit" is set to false in the queue function for following events
    // Otherwise you run a risk of infinite loop.
    compute$5();
    compute$4();
    compute$9();
    compute$7();
    // Treat this as the last payload only if final boolean was explicitly set to true.
    // In real world tests, we noticed that certain third party scripts (e.g. https://www.npmjs.com/package/raven-js)
    // could inject function arguments for internal tracking (likely stack traces for script errors).
    // For these edge cases, we want to ensure that an injected object (e.g. {"key": "value"}) isn't mistaken to be true.
    let last = final === true;
    let e = JSON.stringify(envelope(last));
    let a = `[${analysis.join()}]`;
    let p = sendPlaybackBytes ? `[${playback.join()}]` : "" /* Constant.Empty */;
    let encoded = { e, a, p };
    // Get the payload ready for sending over the wire
    // We also attempt to compress the payload if it is not the last payload and the browser supports it
    // In all other cases, we continue to send back string value
    let payload = stringify(encoded);
    let zipped = last ? null : await compress(payload);
    sum(2 /* Metric.TotalBytes */, zipped ? zipped.length : payload.length);
    send(payload, zipped, data$1.sequence, last);
    // Clear out events now that payload has been dispatched
    analysis = [];
    if (sendPlaybackBytes) {
        playback = [];
        playbackBytes = 0;
        discoverBytes = 0;
    }
}
function stringify(encoded) {
    return encoded.p.length > 0 ? `{"e":${encoded.e},"a":${encoded.a},"p":${encoded.p}}` : `{"e":${encoded.e},"a":${encoded.a}}`;
}
function send(payload, zipped, sequence, beacon = false) {
    // Upload data if a valid URL is defined in the config
    if (typeof config$2.upload === "string" /* Constant.String */) {
        const url = config$2.upload;
        let dispatched = false;
        // If it's the last payload, attempt to upload using sendBeacon first.
        // The advantage to using sendBeacon is that browser can decide to upload asynchronously, improving chances of success
        // However, we don't want to rely on it for every payload, since we have no ability to retry if the upload failed.
        // Also, in case of sendBeacon, we do not have a way to alter HTTP headers and therefore can't send compressed payload
        if (beacon && "sendBeacon" in navigator) {
            try {
                // Navigator needs to be bound to sendBeacon before it is used to avoid errors in some browsers
                dispatched = navigator.sendBeacon.bind(navigator)(url, payload);
                if (dispatched) {
                    done(sequence);
                }
            }
            catch ( /* do nothing - and we will automatically fallback to XHR below */_a) { /* do nothing - and we will automatically fallback to XHR below */ }
        }
        // Before initiating XHR upload, we check if the data has already been uploaded using sendBeacon
        // There are two cases when dispatched could still be false:
        //   a) It's not the last payload, and therefore we didn't attempt sending sendBeacon
        //   b) It's the last payload, however, we failed to queue sendBeacon call and need to now fall back to XHR.
        //      E.g. if data is over 64KB, several user agents (like Chrome) will reject to queue the sendBeacon call.
        if (dispatched === false) {
            // While tracking payload for retry, we only track string value of the payload to err on the safe side
            // Not all browsers support compression API and the support for it in supported browsers is still experimental
            if (sequence in transit) {
                transit[sequence].attempts++;
            }
            else {
                transit[sequence] = { data: payload, attempts: 1 };
            }
            let xhr = new XMLHttpRequest();
            xhr.open("POST", url, true);
            xhr.timeout = 15000 /* Setting.UploadTimeout */;
            xhr.ontimeout = () => { report(new Error(`${"Timeout" /* Constant.Timeout */} : ${url}`)); };
            if (sequence !== null) {
                xhr.onreadystatechange = () => { measure(check$3)(xhr, sequence); };
            }
            xhr.withCredentials = true;
            if (zipped) {
                // If we do have valid compressed array, send it with appropriate HTTP headers so server can decode it appropriately
                xhr.setRequestHeader("Accept" /* Constant.Accept */, "application/x-clarity-gzip" /* Constant.ClarityGzip */);
                xhr.send(zipped);
            }
            else {
                // In all other cases, continue sending string back to the server
                xhr.send(payload);
            }
        }
    }
    else if (config$2.upload) {
        const callback = config$2.upload;
        callback(payload);
        done(sequence);
    }
}
function check$3(xhr, sequence) {
    var transitData = transit[sequence];
    if (xhr && xhr.readyState === 4 /* XMLReadyState.Done */ && transitData) {
        // Attempt send payload again (as configured in settings) if we do not receive a success (2XX) response code back from the server
        if ((xhr.status < 200 || xhr.status > 208) && transitData.attempts <= 1 /* Setting.RetryLimit */) {
            // We re-attempt in all cases except when server explicitly rejects our request with 4XX error
            if (xhr.status >= 400 && xhr.status < 500) {
                // In case of a 4XX response from the server, we bail out instead of trying again
                trigger(6 /* Check.Server */);
            }
            else {
                // Browser will send status = 0 when it refuses to put network request over the wire
                // This could happen for several reasons, couple of known ones are:
                //    1: Browsers block upload because of content security policy violation
                //    2: Safari will terminate pending XHR requests with status code 0 if the user navigates away from the page
                // In any case, we switch the upload URL to fallback configuration (if available) before re-trying one more time
                if (xhr.status === 0) {
                    config$2.upload = config$2.fallback ? config$2.fallback : config$2.upload;
                }
                // In all other cases, re-attempt sending the same data
                // For retry we always fallback to string payload, even though we may have attempted
                // sending zipped payload earlier
                send(transitData.data, null, sequence);
            }
        }
        else {
            track$1 = { sequence, attempts: transitData.attempts, status: xhr.status };
            // Send back an event only if we were not successful in our first attempt
            if (transitData.attempts > 1) {
                encode$1(2 /* Event.Upload */);
            }
            // Handle response if it was a 200 response with a valid body
            if (xhr.status === 200 && xhr.responseText) {
                response(xhr.responseText);
            }
            // If we exhausted our retries then trigger Clarity's shutdown for this page since the data will be incomplete
            if (xhr.status === 0) {
                // And, right before we terminate the session, we will attempt one last time to see if we can use
                // different transport option (sendBeacon vs. XHR) to get this data to the server for analysis purposes
                send(transitData.data, null, sequence, true);
                trigger(3 /* Check.Retry */);
            }
            // Signal that this request completed successfully
            if (xhr.status >= 200 && xhr.status <= 208) {
                done(sequence);
            }
            // Stop tracking this payload now that it's all done
            delete transit[sequence];
        }
    }
}
function done(sequence) {
    // If we everything went successfully, and it is the first sequence, save this session for future reference
    if (sequence === 1) {
        save();
        callback();
    }
}
function delay() {
    // Progressively increase delay as we continue to send more payloads from the client to the server
    // If we are not uploading data to a server, and instead invoking UploadCallback, in that case keep returning configured value
    let gap = config$2.lean === false && discoverBytes > 0 ? 100 /* Setting.MinUploadDelay */ : data$1.sequence * config$2.delay;
    return typeof config$2.upload === "string" /* Constant.String */ ? Math.max(Math.min(gap, 30000 /* Setting.MaxUploadDelay */), 100 /* Setting.MinUploadDelay */) : config$2.delay;
}
function response(payload) {
    let lines = payload && payload.length > 0 ? payload.split("\n") : [];
    for (var line of lines) {
        let parts = line && line.length > 0 ? line.split(/ (.*)/) : ["" /* Constant.Empty */];
        switch (parts[0]) {
            case "END" /* Constant.End */:
                // Clear out session storage and end the session so we can start fresh the next time
                trigger(6 /* Check.Server */);
                break;
            case "UPGRADE" /* Constant.Upgrade */:
                // Upgrade current session to send back playback information
                upgrade("Auto" /* Constant.Auto */);
                break;
            case "ACTION" /* Constant.Action */:
                // Invoke action callback, if configured and has a valid value
                if (config$2.action && parts.length > 1) {
                    config$2.action(parts[1]);
                }
                break;
            case "EXTRACT" /* Constant.Extract */:
                if (parts.length > 1) {
                    trigger$2(parts[1]);
                }
                break;
            case "SIGNAL" /* Constant.Signal */:
                if (parts.length > 1) {
                    signalsEvent(parts[1]);
                }
                break;
        }
    }
}

let history$3 = {};
let data$7;
function start$f() {
    bind(window, "error", handler);
    history$3 = {};
}
function handler(error) {
    handler.dn = 4 /* FunctionNames.ScriptHandler */;
    let e = error["error"] || error;
    // While rare, it's possible for code to fail repeatedly during the lifetime of the same page
    // In those cases, we only want to log the failure first few times and not spam logs with redundant information.
    if (!(e.message in history$3)) {
        history$3[e.message] = 0;
    }
    if (history$3[e.message]++ >= 5 /* Setting.ScriptErrorLimit */) {
        return true;
    }
    // Send back information only if the handled error has valid information
    if (e && e.message) {
        data$7 = {
            message: e.message,
            line: error["lineno"],
            column: error["colno"],
            stack: e.stack,
            source: error["filename"]
        };
        encode$2(31 /* Event.ScriptError */);
    }
    return true;
}

async function encode$2 (type) {
    let tokens = [time(), type];
    switch (type) {
        case 31 /* Event.ScriptError */:
            tokens.push(data$7.message);
            tokens.push(data$7.line);
            tokens.push(data$7.column);
            tokens.push(data$7.stack);
            tokens.push(url$1(data$7.source));
            queue(tokens);
            break;
        case 33 /* Event.Log */:
            if (data$6) {
                tokens.push(data$6.code);
                tokens.push(data$6.name);
                tokens.push(data$6.message);
                tokens.push(data$6.stack);
                tokens.push(data$6.severity);
                queue(tokens, false);
            }
            break;
        case 41 /* Event.Fraud */:
            if (data$d) {
                tokens.push(data$d.id);
                tokens.push(data$d.target);
                tokens.push(data$d.checksum);
                queue(tokens, false);
            }
            break;
    }
}

let history$2 = {};
let data$6;
function start$e() {
    history$2 = {};
}
function log$1(code, severity, name = null, message = null, stack = null) {
    let key = name ? `${name}|${message}` : "";
    // While rare, it's possible for code to fail repeatedly during the lifetime of the same page
    // In those cases, we only want to log the failure once and not spam logs with redundant information.
    if (code in history$2 && history$2[code].indexOf(key) >= 0) {
        return;
    }
    data$6 = { code, name, message, stack, severity };
    // Maintain history of errors in memory to avoid sending redundant information
    if (code in history$2) {
        history$2[code].push(key);
    }
    else {
        history$2[code] = [key];
    }
    encode$2(33 /* Event.Log */);
}
function stop$d() {
    history$2 = {};
}

// Track the start time to be able to compute duration at the end of the task
const idleTimeout = 5000;
let tracker = {};
let queuedTasks = [];
let activeTask = null;
let pauseTask = null;
let resumeResolve = null;
function pause$1() {
    if (pauseTask === null) {
        pauseTask = new Promise((resolve) => {
            resumeResolve = resolve;
        });
    }
}
function resume$1() {
    if (pauseTask) {
        resumeResolve();
        pauseTask = null;
        if (activeTask === null) {
            run();
        }
    }
}
function reset$4() {
    tracker = {};
    queuedTasks = [];
    activeTask = null;
    pauseTask = null;
}
async function schedule(task, priority = 0 /* Priority.Normal */) {
    // If this task is already scheduled, skip it
    for (let q of queuedTasks) {
        if (q.task === task) {
            return;
        }
    }
    let promise = new Promise((resolve) => {
        let insert = priority === 1 /* Priority.High */ ? "unshift" : "push";
        // Queue this task for asynchronous execution later
        // We also store a unique page identifier (id) along with the task to ensure
        // ensure that we do not accidentally execute this task in context of a different page
        queuedTasks[insert]({ task, resolve, id: id() });
    });
    // If there is no active task running, and Clarity is not in pause state,
    // invoke the first task in the queue synchronously. This ensures that we don't yield the thread during unload event
    if (activeTask === null && pauseTask === null) {
        run();
    }
    return promise;
}
function run() {
    let entry = queuedTasks.shift();
    if (entry) {
        activeTask = entry;
        entry.task().then(() => {
            // Bail out if the context in which this task was operating is different from the current page
            // An example scenario where task could span across pages is Single Page Applications (SPA)
            // A task that started on page #1, but completes on page #2
            if (entry.id !== id()) {
                return;
            }
            entry.resolve();
            activeTask = null; // Reset active task back to null now that the promise is resolved
            run();
        }).catch((error) => {
            // If one of the scheduled tasks failed, log, recover and continue processing rest of the tasks
            if (entry.id !== id()) {
                return;
            }
            if (error) {
                log$1(0 /* Code.RunTask */, 1 /* Severity.Warning */, error.name, error.message, error.stack);
            }
            activeTask = null;
            run();
        });
    }
}
function state(timer) {
    let id = key(timer);
    if (id in tracker) {
        let elapsed = performance.now() - tracker[id].start;
        return (elapsed > tracker[id].yield) ? 0 /* Task.Wait */ : 1 /* Task.Run */;
    }
    // If this task is no longer being tracked, send stop message to the caller
    return 2 /* Task.Stop */;
}
function start$d(timer) {
    tracker[key(timer)] = { start: performance.now(), calls: 0, yield: 30 /* Setting.LongTask */ };
}
function restart$2(timer) {
    let id = key(timer);
    if (tracker && tracker[id]) {
        let c = tracker[id].calls;
        let y = tracker[id].yield;
        start$d(timer);
        tracker[id].calls = c + 1;
        tracker[id].yield = y;
    }
}
function stop$c(timer) {
    let end = performance.now();
    let id = key(timer);
    let duration = end - tracker[id].start;
    sum(timer.cost, duration);
    count$1(5 /* Metric.InvokeCount */);
    // For the first execution, which is synchronous, time is automatically counted towards TotalDuration.
    // However, for subsequent asynchronous runs, we need to manually update TotalDuration metric.
    if (tracker[id].calls > 0) {
        sum(4 /* Metric.TotalCost */, duration);
    }
}
async function suspend$1(timer) {
    var _a;
    // Suspend and yield the thread only if the task is still being tracked
    // It's possible that Clarity is wrapping up instrumentation on a page and we are still in the middle of an async task.
    // In that case, we do not wish to continue yielding thread.
    // Instead, we will turn async task into a sync task and maximize our chances of getting some data back.
    let id = key(timer);
    if (id in tracker) {
        stop$c(timer);
        // some customer polyfills for requestIdleCallback return null 
        tracker[id].yield = ((_a = (await wait())) === null || _a === void 0 ? void 0 : _a.timeRemaining()) || 30 /* Setting.LongTask */;
        restart$2(timer);
    }
    // After we are done with suspending task, ensure that we are still operating in the right context
    // If the task is still being tracked, continue running the task, otherwise ask caller to stop execution
    return id in tracker ? 1 /* Task.Run */ : 2 /* Task.Stop */;
}
function key(timer) {
    return `${timer.id}.${timer.cost}`;
}
async function wait() {
    if (pauseTask) {
        await pauseTask;
    }
    return new Promise((resolve) => {
        requestIdleCallback(resolve, { timeout: idleTimeout });
    });
}
// Use native implementation of requestIdleCallback if it exists.
// Otherwise, fall back to a custom implementation using requestAnimationFrame & MessageChannel.
// While it's not possible to build a perfect polyfill given the nature of this API, the following code attempts to get close.
// Background context: requestAnimationFrame invokes the js code right before: style, layout and paint computation within the frame.
// This means, that any code that runs as part of requestAnimationFrame will by default be blocking in nature. Not what we want.
// For non-blocking behavior, We need to know when browser has finished painting. This can be accomplished in two different ways (hacks):
//   (1) Use MessageChannel to pass the message, and browser will receive the message right after paint event has occured.
//   (2) Use setTimeout call within requestAnimationFrame. This also works, but there's a risk that browser may throttle setTimeout calls.
// Given this information, we are currently using (1) from above. More information on (2) as well as some additional context is below:
// https://developer.mozilla.org/en-US/docs/Mozilla/Firefox/Performance_best_practices_for_Firefox_fe_engineers
function requestIdleCallbackPolyfill(callback, options) {
    const startTime = performance.now();
    const channel = new MessageChannel();
    const incoming = channel.port1;
    const outgoing = channel.port2;
    incoming.onmessage = (event) => {
        let currentTime = performance.now();
        let elapsed = currentTime - startTime;
        let duration = currentTime - event.data;
        if (duration > 30 /* Setting.LongTask */ && elapsed < options.timeout) {
            requestAnimationFrame(() => { outgoing.postMessage(currentTime); });
        }
        else {
            let didTimeout = elapsed > options.timeout;
            callback({
                didTimeout,
                timeRemaining: () => didTimeout ? 30 /* Setting.LongTask */ : Math.max(0, 30 /* Setting.LongTask */ - duration)
            });
        }
    };
    requestAnimationFrame(() => { outgoing.postMessage(performance.now()); });
}
let requestIdleCallback = window["requestIdleCallback"] || requestIdleCallbackPolyfill;

function start$c() {
    schedule(discover, 1 /* Priority.High */).then(() => {
        measure(compute$6)();
        measure(compute$5)();
        measure(compute$8)();
    });
}
async function discover() {
    let ts = time();
    let timer = { id: id(), cost: 3 /* Metric.LayoutCost */ };
    start$d(timer);
    await traverse(document, timer, 0 /* Source.Discover */, ts);
    checkDocumentStyles(document, ts);
    await encode$4(5 /* Event.Discover */, timer, ts);
    stop$c(timer);
}

let data$5 = null;
function start$b() {
    if (!config$2.lean && config$2.upgrade) {
        config$2.upgrade("Config" /* Constant.Config */);
    }
    data$5 = null;
}
// Following call will upgrade the session from lean mode into the full mode retroactively from the start of the page.
// As part of the lean mode, we do not send back any layout information - including discovery of DOM and mutations.
// However, if there's a need for full fidelity playback, calling this function will disable lean mode
// and send all backed up layout events to the server.
function upgrade(key) {
    // Upgrade only if Clarity was successfully activated on the page
    if (active() && config$2.lean) {
        config$2.lean = false;
        data$5 = { key };
        // Update metadata to track we have upgraded this session
        callback();
        save();
        // Callback upgrade handler, if configured
        if (config$2.upgrade) {
            config$2.upgrade(key);
        }
        encode$1(3 /* Event.Upgrade */);
        if (config$2.lite) {
            start$c();
            start$n();
        }
    }
}
function stop$b() {
    data$5 = null;
}

function encode$1 (event) {
    let t = time();
    let tokens = [t, event];
    switch (event) {
        case 4 /* Event.Baseline */:
            let b = state$b;
            if (b) {
                tokens = [b.time, b.event];
                tokens.push(b.data.visible);
                tokens.push(b.data.docWidth);
                tokens.push(b.data.docHeight);
                tokens.push(b.data.screenWidth);
                tokens.push(b.data.screenHeight);
                tokens.push(b.data.scrollX);
                tokens.push(b.data.scrollY);
                tokens.push(b.data.pointerX);
                tokens.push(b.data.pointerY);
                tokens.push(b.data.activityTime);
                tokens.push(b.data.scrollTime);
                tokens.push(b.data.pointerTime);
                tokens.push(b.data.moveX);
                tokens.push(b.data.moveY);
                tokens.push(b.data.moveTime);
                tokens.push(b.data.downX);
                tokens.push(b.data.downY);
                tokens.push(b.data.downTime);
                tokens.push(b.data.upX);
                tokens.push(b.data.upY);
                tokens.push(b.data.upTime);
                tokens.push(b.data.pointerPrevX);
                tokens.push(b.data.pointerPrevY);
                tokens.push(b.data.pointerPrevTime);
                queue(tokens, false);
            }
            reset$s();
            break;
        case 25 /* Event.Ping */:
            tokens.push(data$h.gap);
            queue(tokens);
            break;
        case 35 /* Event.Limit */:
            tokens.push(data$4.check);
            queue(tokens, false);
            break;
        case 3 /* Event.Upgrade */:
            tokens.push(data$5.key);
            queue(tokens);
            break;
        case 2 /* Event.Upload */:
            tokens.push(track$1.sequence);
            tokens.push(track$1.attempts);
            tokens.push(track$1.status);
            queue(tokens, false);
            break;
        case 24 /* Event.Custom */:
            // not all custom events have a key - if it wasn't passed server handles just value
            data$j.key && tokens.push(data$j.key);
            tokens.push(data$j.value);
            queue(tokens);
            break;
        case 34 /* Event.Variable */:
            let variableKeys = Object.keys(data$f);
            if (variableKeys.length > 0) {
                for (let v of variableKeys) {
                    tokens.push(v);
                    tokens.push(data$f[v]);
                }
                reset$o();
                queue(tokens, false);
            }
            break;
        case 0 /* Event.Metric */:
            let metricKeys = Object.keys(updates$3);
            if (metricKeys.length > 0) {
                for (let m of metricKeys) {
                    let key = parseInt(m, 10);
                    tokens.push(key);
                    // For computation, we need microseconds precision that performance.now() API offers
                    // However, for data over the wire, we round it off to milliseconds precision.
                    tokens.push(Math.round(updates$3[m]));
                }
                reset$r();
                queue(tokens, false);
            }
            break;
        case 1 /* Event.Dimension */:
            let dimensionKeys = Object.keys(updates);
            if (dimensionKeys.length > 0) {
                for (let d of dimensionKeys) {
                    let key = parseInt(d, 10);
                    tokens.push(key);
                    tokens.push(updates[d]);
                }
                reset$3();
                queue(tokens, false);
            }
            break;
        case 36 /* Event.Summary */:
            let eventKeys = Object.keys(data$g);
            if (eventKeys.length > 0) {
                for (let e of eventKeys) {
                    let key = parseInt(e, 10);
                    tokens.push(key);
                    tokens.push([].concat(...data$g[e]));
                }
                reset$p();
                queue(tokens, false);
            }
            break;
        case 40 /* Event.Extract */:
            let extractKeys = keys;
            extractKeys.forEach((e => {
                tokens.push(e);
                let token = [];
                for (let d in data$e[e]) {
                    let key = parseInt(d, 10);
                    token.push(key);
                    token.push(data$e[e][d]);
                }
                tokens.push(token);
            }));
            reset$n();
            queue(tokens, false);
    }
}

let data$4;
function start$a() {
    data$4 = { check: 0 /* Check.None */ };
}
function check$2(bytes) {
    if (data$4.check === 0 /* Check.None */) {
        let reason = data$4.check;
        reason = data$1.sequence >= 128 /* Setting.PayloadLimit */ ? 1 /* Check.Payload */ : reason;
        reason = data$1.pageNum >= 128 /* Setting.PageLimit */ ? 7 /* Check.Page */ : reason;
        reason = time() > 7200000 /* Setting.ShutdownLimit */ ? 2 /* Check.Shutdown */ : reason;
        reason = bytes > 10485760 /* Setting.PlaybackBytesLimit */ ? 2 /* Check.Shutdown */ : reason;
        if (reason !== data$4.check) {
            trigger(reason);
        }
    }
}
function trigger(reason) {
    data$4.check = reason;
    // limit the dimensions we collect, but we don't need to stop Clarity entirely if we hit the limit
    if (reason !== 5 /* Check.Collection */) {
        clear();
        stop();
    }
}
function compute$3() {
    if (data$4.check !== 0 /* Check.None */) {
        encode$1(35 /* Event.Limit */);
    }
}
function stop$a() {
    data$4 = null;
}

let data$3 = null;
let updates = null;
let limited = false;
function start$9() {
    data$3 = {};
    updates = {};
    limited = false;
}
function stop$9() {
    data$3 = {};
    updates = {};
    limited = false;
}
function log(dimension, value) {
    // Check valid value before moving ahead
    if (value) {
        // Ensure received value is casted into a string if it wasn't a string to begin with
        value = `${value}`;
        if (!(dimension in data$3)) {
            data$3[dimension] = [];
        }
        if (data$3[dimension].indexOf(value) < 0) {
            // Limit check to ensure we have a cap on number of dimensions we can collect
            if (data$3[dimension].length > 128 /* Setting.CollectionLimit */) {
                if (!limited) {
                    limited = true;
                    trigger(5 /* Check.Collection */);
                }
                return;
            }
            data$3[dimension].push(value);
            // If this is a new value, track it as part of updates object
            // This allows us to only send back new values in subsequent payloads
            if (!(dimension in updates)) {
                updates[dimension] = [];
            }
            updates[dimension].push(value);
        }
    }
}
function compute$2() {
    encode$1(1 /* Event.Dimension */);
}
function reset$3() {
    updates = {};
    limited = false;
}

function config$1(track) {
    trackConsent(track ? 1 /* ConsentType.Implicit */ : 0 /* ConsentType.None */);
}
// When we get consent signal as false, we restart the service and track config as false.
function consent$1() {
    trackConsent(2 /* ConsentType.General */);
}
function trackConsent(consent) {
    log(36 /* Dimension.Consent */, consent.toString());
}

let data$2 = null;
let callbacks = [];
let electron = 0 /* BooleanFlag.False */;
let rootDomain = null;
function start$8() {
    var _a, _b, _c;
    rootDomain = null;
    const ua = navigator && "userAgent" in navigator ? navigator.userAgent : "" /* Constant.Empty */;
    const timezone = (_c = (_b = (_a = Intl === null || Intl === void 0 ? void 0 : Intl.DateTimeFormat()) === null || _a === void 0 ? void 0 : _a.resolvedOptions()) === null || _b === void 0 ? void 0 : _b.timeZone) !== null && _c !== void 0 ? _c : '';
    const timezoneOffset = new Date().getTimezoneOffset().toString();
    const ancestorOrigins = window.location.ancestorOrigins ? Array.from(window.location.ancestorOrigins).toString() : '';
    const title = document && document.title ? document.title : "" /* Constant.Empty */;
    electron = ua.indexOf("Electron" /* Constant.Electron */) > 0 ? 1 /* BooleanFlag.True */ : 0 /* BooleanFlag.False */;
    // Populate ids for this page
    let s = session();
    let u = user();
    let projectId = config$2.projectId || hash(location.host);
    data$2 = { projectId, userId: u.id, sessionId: s.session, pageNum: s.count };
    // Override configuration based on what's in the session storage, unless it is blank (e.g. using upload callback, like in devtools)
    config$2.lean = config$2.track && s.upgrade !== null ? s.upgrade === 0 /* BooleanFlag.False */ : config$2.lean;
    config$2.upload = config$2.track && typeof config$2.upload === "string" /* Constant.String */ && s.upload && s.upload.length > "https://" /* Constant.HTTPS */.length ? s.upload : config$2.upload;
    // Log page metadata as dimensions
    log(0 /* Dimension.UserAgent */, ua);
    log(3 /* Dimension.PageTitle */, title);
    log(1 /* Dimension.Url */, url$1(location.href, !!electron));
    log(2 /* Dimension.Referrer */, document.referrer);
    log(15 /* Dimension.TabId */, tab());
    log(16 /* Dimension.PageLanguage */, document.documentElement.lang);
    log(17 /* Dimension.DocumentDirection */, document.dir);
    log(26 /* Dimension.DevicePixelRatio */, `${window.devicePixelRatio}`);
    log(28 /* Dimension.Dob */, u.dob.toString());
    log(29 /* Dimension.CookieVersion */, u.version.toString());
    log(33 /* Dimension.AncestorOrigins */, ancestorOrigins);
    log(34 /* Dimension.Timezone */, timezone);
    log(35 /* Dimension.TimezoneOffset */, timezoneOffset);
    // Capture additional metadata as metrics
    max(0 /* Metric.ClientTimestamp */, s.ts);
    max(1 /* Metric.Playback */, 0 /* BooleanFlag.False */);
    max(35 /* Metric.Electron */, electron);
    // Capture navigator specific dimensions
    if (navigator) {
        log(9 /* Dimension.Language */, navigator.language);
        max(33 /* Metric.HardwareConcurrency */, navigator.hardwareConcurrency);
        max(32 /* Metric.MaxTouchPoints */, navigator.maxTouchPoints);
        max(34 /* Metric.DeviceMemory */, Math.round(navigator.deviceMemory));
        userAgentData();
    }
    if (screen) {
        max(14 /* Metric.ScreenWidth */, Math.round(screen.width));
        max(15 /* Metric.ScreenHeight */, Math.round(screen.height));
        max(16 /* Metric.ColorDepth */, Math.round(screen.colorDepth));
    }
    // Read cookies specified in configuration
    for (let key of config$2.cookies) {
        let value = getCookie(key);
        if (value) {
            set(key, value);
        }
    }
    // Track consent config
    config$1(config$2.track);
    // Track ids using a cookie if configuration allows it
    track(u);
}
function userAgentData() {
    let uaData = navigator["userAgentData"];
    if (uaData && uaData.getHighEntropyValues) {
        uaData.getHighEntropyValues(["model", "platform", "platformVersion", "uaFullVersion"]).then(ua => {
            var _a;
            log(22 /* Dimension.Platform */, ua.platform);
            log(23 /* Dimension.PlatformVersion */, ua.platformVersion);
            (_a = ua.brands) === null || _a === void 0 ? void 0 : _a.forEach(brand => { log(24 /* Dimension.Brand */, brand.name + "~" /* Constant.Tilde */ + brand.version); });
            log(25 /* Dimension.Model */, ua.model);
            max(27 /* Metric.Mobile */, ua.mobile ? 1 /* BooleanFlag.True */ : 0 /* BooleanFlag.False */);
        });
    }
    else {
        log(22 /* Dimension.Platform */, navigator.platform);
    }
}
function stop$8() {
    rootDomain = null;
    data$2 = null;
    callbacks.forEach(cb => { cb.called = false; });
}
function metadata(cb, wait = true, recall = false) {
    let upgraded = config$2.lean ? 0 /* BooleanFlag.False */ : 1 /* BooleanFlag.True */;
    let called = false;
    // if caller hasn't specified that they want to skip waiting for upgrade but we've already upgraded, we need to
    // directly execute the callback in addition to adding to our list as we only process callbacks at the moment
    // we go through the upgrading flow.
    if (data$2 && (upgraded || wait === false)) {
        // Immediately invoke the callback if the caller explicitly doesn't want to wait for the upgrade confirmation
        cb(data$2, !config$2.lean);
        called = true;
    }
    if (recall || !called) {
        callbacks.push({ callback: cb, wait, recall, called });
    }
}
function id() {
    return data$2 ? [data$2.userId, data$2.sessionId, data$2.pageNum].join("." /* Constant.Dot */) : "" /* Constant.Empty */;
}
function consent(status = true) {
    if (!status) {
        config$2.track = false;
        setCookie("_clsk" /* Constant.SessionKey */, "" /* Constant.Empty */, -Number.MAX_VALUE);
        setCookie("_clck" /* Constant.CookieKey */, "" /* Constant.Empty */, -Number.MAX_VALUE);
        stop();
        window.setTimeout(start, 250 /* Setting.RestartDelay */);
        return;
    }
    if (active()) {
        config$2.track = true;
        track(user(), 1 /* BooleanFlag.True */);
        save();
        consent$1();
    }
}
function clear() {
    // Clear any stored information in the cookie that tracks session information so we can restart fresh the next time
    setCookie("_clsk" /* Constant.SessionKey */, "" /* Constant.Empty */, 0);
}
function tab() {
    let id = shortid();
    if (config$2.track && supported(window, "sessionStorage" /* Constant.SessionStorage */)) {
        let value = sessionStorage.getItem("_cltk" /* Constant.TabKey */);
        id = value ? value : id;
        sessionStorage.setItem("_cltk" /* Constant.TabKey */, id);
    }
    return id;
}
function callback() {
    let upgrade = config$2.lean ? 0 /* BooleanFlag.False */ : 1 /* BooleanFlag.True */;
    processCallback(upgrade);
}
function save() {
    if (!data$2 || !config$2.track)
        return;
    let ts = Math.round(Date.now());
    let upload = config$2.upload && typeof config$2.upload === "string" /* Constant.String */ ? config$2.upload.replace("https://" /* Constant.HTTPS */, "" /* Constant.Empty */) : "" /* Constant.Empty */;
    let upgrade = config$2.lean ? 0 /* BooleanFlag.False */ : 1 /* BooleanFlag.True */;
    setCookie("_clsk" /* Constant.SessionKey */, [data$2.sessionId, ts, data$2.pageNum, upgrade, upload].join("|" /* Constant.Pipe */), 1 /* Setting.SessionExpire */);
}
function processCallback(upgrade) {
    if (callbacks.length > 0) {
        for (let i = 0; i < callbacks.length; i++) {
            const cb = callbacks[i];
            if (cb.callback && !cb.called && (!cb.wait || upgrade)) {
                cb.callback(data$2, !config$2.lean);
                cb.called = true;
                if (!cb.recall) {
                    callbacks.splice(i, 1);
                    i--;
                }
            }
        }
    }
}
function supported(target, api) {
    try {
        return !!target[api];
    }
    catch (_a) {
        return false;
    }
}
function track(u, consent = null) {
    // If consent is not explicitly specified, infer it from the user object
    consent = consent === null ? u.consent : consent;
    // Convert time precision into days to reduce number of bytes we have to write in a cookie
    // E.g. Math.ceil(1628735962643 / (24*60*60*1000)) => 18852 (days) => ejo in base36 (13 bytes => 3 bytes)
    let end = Math.ceil((Date.now() + (365 /* Setting.Expire */ * 86400000 /* Time.Day */)) / 86400000 /* Time.Day */);
    // If DOB is not set in the user object, use the date set in the config as a DOB
    let dob = u.dob === 0 ? (config$2.dob === null ? 0 : config$2.dob) : u.dob;
    // To avoid cookie churn, write user id cookie only once every day
    if (u.expiry === null || Math.abs(end - u.expiry) >= 1 /* Setting.CookieInterval */ || u.consent !== consent || u.dob !== dob) {
        let cookieParts = [data$2.userId, 2 /* Setting.CookieVersion */, end.toString(36), consent, dob];
        setCookie("_clck" /* Constant.CookieKey */, cookieParts.join("|" /* Constant.Pipe */), 365 /* Setting.Expire */);
    }
}
function shortid() {
    let id = Math.floor(Math.random() * Math.pow(2, 32));
    if (window && window.crypto && window.crypto.getRandomValues && Uint32Array) {
        id = window.crypto.getRandomValues(new Uint32Array(1))[0];
    }
    return id.toString(36);
}
function session() {
    let output = { session: shortid(), ts: Math.round(Date.now()), count: 1, upgrade: null, upload: "" /* Constant.Empty */ };
    let value = getCookie("_clsk" /* Constant.SessionKey */, !config$2.includeSubdomains);
    if (value) {
        // Maintaining support for pipe separator for backward compatibility, this can be removed in future releases
        let parts = value.includes("^" /* Constant.Caret */) ? value.split("^" /* Constant.Caret */) : value.split("|" /* Constant.Pipe */);
        // Making it backward & forward compatible by using greater than comparison (v0.6.21)
        // In future version, we can reduce the parts length to be 5 where the last part contains the full upload URL
        if (parts.length >= 5 && output.ts - num(parts[1]) < 1800000 /* Setting.SessionTimeout */) {
            output.session = parts[0];
            output.count = num(parts[2]) + 1;
            output.upgrade = num(parts[3]);
            output.upload = parts.length >= 6 ? `${"https://" /* Constant.HTTPS */}${parts[5]}/${parts[4]}` : `${"https://" /* Constant.HTTPS */}${parts[4]}`;
        }
    }
    return output;
}
function num(string, base = 10) {
    return parseInt(string, base);
}
function user() {
    let output = { id: shortid(), version: 0, expiry: null, consent: 0 /* BooleanFlag.False */, dob: 0 };
    let cookie = getCookie("_clck" /* Constant.CookieKey */, !config$2.includeSubdomains);
    if (cookie && cookie.length > 0) {
        // Splitting and looking up first part for forward compatibility, in case we wish to store additional information in a cookie
        // Maintaining support for pipe separator for backward compatibility, this can be removed in future releases
        let parts = cookie.includes("^" /* Constant.Caret */) ? cookie.split("^" /* Constant.Caret */) : cookie.split("|" /* Constant.Pipe */);
        // Read version information and timestamp from cookie, if available
        if (parts.length > 1) {
            output.version = num(parts[1]);
        }
        if (parts.length > 2) {
            output.expiry = num(parts[2], 36);
        }
        // Check if we have explicit consent to track this user
        if (parts.length > 3 && num(parts[3]) === 1) {
            output.consent = 1 /* BooleanFlag.True */;
        }
        if (parts.length > 4 && num(parts[1]) > 1) {
            output.dob = num(parts[4]);
        }
        // Set track configuration to true for this user if we have explicit consent, regardless of project setting
        config$2.track = config$2.track || output.consent === 1 /* BooleanFlag.True */;
        // Get user id from cookie only if we tracking is enabled, otherwise fallback to a random id
        output.id = config$2.track ? parts[0] : output.id;
    }
    return output;
}
function getCookie(key, limit = false) {
    if (supported(document, "cookie" /* Constant.Cookie */)) {
        let cookies = document.cookie.split(";" /* Constant.Semicolon */);
        if (cookies) {
            for (let i = 0; i < cookies.length; i++) {
                let pair = cookies[i].split("=" /* Constant.Equals */);
                if (pair.length > 1 && pair[0] && pair[0].trim() === key) {
                    // Some browsers automatically url encode cookie values if they are not url encoded.
                    // We therefore encode and decode cookie values ourselves.
                    // For backwards compatability we need to consider 3 cases:
                    // * Cookie was previously not encoded by Clarity and browser did not encode it
                    // * Cookie was previously not encoded by Clarity and browser encoded it once or more
                    // * Cookie was previously encoded by Clarity and browser did not encode it
                    let [isEncoded, decodedValue] = decodeCookieValue(pair[1]);
                    while (isEncoded) {
                        [isEncoded, decodedValue] = decodeCookieValue(decodedValue);
                    }
                    // If we are limiting cookies, check if the cookie value is limited
                    if (limit) {
                        return decodedValue.endsWith(`${"~" /* Constant.Tilde */}1`)
                            ? decodedValue.substring(0, decodedValue.length - 2)
                            : null;
                    }
                    return decodedValue;
                }
            }
        }
    }
    return null;
}
function decodeCookieValue(value) {
    try {
        let decodedValue = decodeURIComponent(value);
        return [decodedValue != value, decodedValue];
    }
    catch (_a) {
    }
    return [false, value];
}
function encodeCookieValue(value) {
    return encodeURIComponent(value);
}
function setCookie(key, value, time) {
    // only write cookies if we are currently in a cookie writing mode (and they are supported)
    // OR if we are trying to write an empty cookie (i.e. clear the cookie value out)
    if ((config$2.track || value == "" /* Constant.Empty */) && ((navigator && navigator.cookieEnabled) || supported(document, "cookie" /* Constant.Cookie */))) {
        // Some browsers automatically url encode cookie values if they are not url encoded.
        // We therefore encode and decode cookie values ourselves.
        let encodedValue = encodeCookieValue(value);
        let expiry = new Date();
        expiry.setDate(expiry.getDate() + time);
        let expires = expiry ? "expires=" /* Constant.Expires */ + expiry.toUTCString() : "" /* Constant.Empty */;
        let cookie = `${key}=${encodedValue}${";" /* Constant.Semicolon */}${expires}${";path=/" /* Constant.Path */}`;
        try {
            // Attempt to get the root domain only once and fall back to writing cookie on the current domain.
            if (rootDomain === null) {
                let hostname = location.hostname ? location.hostname.split("." /* Constant.Dot */) : [];
                // Walk backwards on a domain and attempt to set a cookie, until successful
                for (let i = hostname.length - 1; i >= 0; i--) {
                    rootDomain = `.${hostname[i]}${rootDomain ? rootDomain : "" /* Constant.Empty */}`;
                    // We do not wish to attempt writing a cookie on the absolute last part of the domain, e.g. .com or .net.
                    // So we start attempting after second-last part, e.g. .domain.com (PASS) or .co.uk (FAIL)
                    if (i < hostname.length - 1) {
                        // Write the cookie on the current computed top level domain
                        document.cookie = `${cookie}${";" /* Constant.Semicolon */}${"domain=" /* Constant.Domain */}${rootDomain}`;
                        // Once written, check if the cookie exists and its value matches exactly with what we intended to set
                        // Checking for exact value match helps us eliminate a corner case where the cookie may already be present with a different value
                        // If the check is successful, no more action is required and we can return from the function since rootDomain cookie is already set
                        // If the check fails, continue with the for loop until we can successfully set and verify the cookie
                        if (getCookie(key) === value) {
                            return;
                        }
                    }
                }
                // Finally, if we were not successful and gone through all the options, play it safe and reset rootDomain to be empty
                // This forces our code to fall back to always writing cookie to the current domain
                rootDomain = "" /* Constant.Empty */;
            }
        }
        catch (_a) {
            rootDomain = "" /* Constant.Empty */;
        }
        document.cookie = rootDomain ? `${cookie}${";" /* Constant.Semicolon */}${"domain=" /* Constant.Domain */}${rootDomain}` : cookie;
    }
}

let data$1 = null;
function start$7() {
    const m = data$2;
    data$1 = {
        version: version$1,
        sequence: 0,
        start: 0,
        duration: 0,
        projectId: m.projectId,
        userId: m.userId,
        sessionId: m.sessionId,
        pageNum: m.pageNum,
        upload: 0 /* Upload.Async */,
        end: 0 /* BooleanFlag.False */,
        applicationPlatform: 0 /* ApplicationPlatform.WebApp */,
        url: ''
    };
}
function stop$7() {
    data$1 = null;
}
function envelope(last) {
    data$1.start = data$1.start + data$1.duration;
    data$1.duration = time() - data$1.start;
    data$1.sequence++;
    data$1.upload = last && "sendBeacon" in navigator ? 1 /* Upload.Beacon */ : 0 /* Upload.Async */;
    data$1.end = last ? 1 /* BooleanFlag.True */ : 0 /* BooleanFlag.False */;
    data$1.applicationPlatform = 0 /* ApplicationPlatform.WebApp */;
    data$1.url = url$1(location.href, false, true);
    return [
        data$1.version,
        data$1.sequence,
        data$1.start,
        data$1.duration,
        data$1.projectId,
        data$1.userId,
        data$1.sessionId,
        data$1.pageNum,
        data$1.upload,
        data$1.end,
        data$1.applicationPlatform,
        data$1.url
    ];
}

let history$1;
function reset$2() {
    history$1 = [];
}
function report(e) {
    // Do not report the same message twice for the same page
    if (history$1 && history$1.indexOf(e.message) === -1) {
        const url = config$2.report;
        if (url && url.length > 0) {
            let payload = { v: data$1.version, p: data$1.projectId, u: data$1.userId, s: data$1.sessionId, n: data$1.pageNum };
            if (e.message) {
                payload.m = e.message;
            }
            if (e.stack) {
                payload.e = e.stack;
            }
            // Using POST request instead of a GET request (img-src) to not violate existing CSP rules
            // Since, Clarity already uses XHR to upload data, we stick with similar POST mechanism for reporting too
            let xhr = new XMLHttpRequest();
            xhr.open("POST", url, true);
            xhr.send(JSON.stringify(payload));
            history$1.push(e.message);
        }
    }
    return e;
}

// tslint:disable-next-line: ban-types
function measure (method) {
    return function () {
        let start = performance.now();
        try {
            method.apply(this, arguments);
        }
        catch (ex) {
            throw report(ex);
        }
        let duration = performance.now() - start;
        sum(4 /* Metric.TotalCost */, duration);
        if (duration > 30 /* Setting.LongTask */) {
            count$1(7 /* Metric.LongTaskCount */);
            max(6 /* Metric.ThreadBlockedTime */, duration);
            log$1(9 /* Code.FunctionExecutionTime */, 0 /* Severity.Info */, `${method.dn || method.name}-${duration}`);
        }
    };
}

let bindings = new Map();
function bind(target, event, listener, capture = false, passive = true) {
    listener = measure(listener);
    // Wrapping following lines inside try / catch to cover edge cases where we might try to access an inaccessible element.
    // E.g. Iframe may start off as same-origin but later turn into cross-origin, and the following lines will throw an exception.
    try {
        target[api("addEventListener" /* Constant.AddEventListener */)](event, listener, { capture, passive });
        if (!has(target)) {
            bindings.set(target, []);
        }
        bindings.get(target).push({ event, listener, options: { capture, passive } });
    }
    catch (_a) {
        /* do nothing */
    }
}
function reset$1() {
    // Walk through existing list of bindings and remove them all
    bindings.forEach((bindingsPerTarget, target) => {
        resetByTarget(bindingsPerTarget, target);
    });
    bindings = new Map();
}
function unbind(target) {
    if (!has(target)) {
        return;
    }
    resetByTarget(bindings.get(target), target);
}
function has(target) {
    return bindings.has(target);
}
function resetByTarget(bindingsPerTarget, target) {
    bindingsPerTarget.forEach((binding) => {
        // Wrapping inside try / catch to avoid situations where the element may be destroyed before we get a chance to unbind
        try {
            target[api("removeEventListener" /* Constant.RemoveEventListener */)](binding.event, binding.listener, { capture: binding.options.capture, passive: binding.options.passive });
        }
        catch (_a) {
            /* do nothing */
        }
    });
    bindings.delete(target);
}

let pushState = null;
let replaceState = null;
let url = null;
let count = 0;
function start$6() {
    url = getCurrentUrl();
    count = 0;
    bind(window, "popstate", compute$1);
    // Add a proxy to history.pushState function
    if (pushState === null) {
        pushState = history.pushState;
        history.pushState = function () {
            pushState.apply(this, arguments);
            if (active() && check$1()) {
                compute$1();
            }
        };
    }
    // Add a proxy to history.replaceState function
    if (replaceState === null) {
        replaceState = history.replaceState;
        history.replaceState = function () {
            replaceState.apply(this, arguments);
            if (active() && check$1()) {
                compute$1();
            }
        };
    }
}
function check$1() {
    if (count++ > 20 /* Setting.CallStackDepth */) {
        log$1(4 /* Code.CallStackDepth */, 0 /* Severity.Info */);
        return false;
    }
    return true;
}
function compute$1() {
    compute$1.dn = 1 /* FunctionNames.HistoryCompute */;
    count = 0; // Reset the counter
    if (url !== getCurrentUrl()) {
        // If the url changed, start tracking it as a new page
        stop();
        window.setTimeout(restart$1, 250 /* Setting.RestartDelay */);
    }
}
function restart$1() {
    start();
    max(29 /* Metric.SinglePage */, 1 /* BooleanFlag.True */);
}
function getCurrentUrl() {
    return location.href ? location.href.replace(location.hash, "" /* Constant.Empty */) : location.href;
}
function stop$6() {
    url = null;
    count = 0;
}

let status = false;
function start$5() {
    status = true;
    start$I();
    reset$4();
    reset$1();
    reset$2();
    start$6();
}
function stop$5() {
    stop$6();
    reset$2();
    reset$1();
    reset$4();
    stop$F();
    status = false;
}
function active() {
    return status;
}
function check() {
    try {
        let globalPrivacyControlSet = navigator && "globalPrivacyControl" in navigator && navigator['globalPrivacyControl'] == true;
        return status === false &&
            typeof Promise !== "undefined" &&
            window["MutationObserver"] &&
            document["createTreeWalker"] &&
            "now" in Date &&
            "now" in performance &&
            typeof WeakMap !== "undefined" &&
            !globalPrivacyControlSet;
    }
    catch (ex) {
        return false;
    }
}
function config(override) {
    // Process custom configuration overrides, if available
    if (override === null || status) {
        return false;
    }
    for (let key in override) {
        if (key in config$2) {
            config$2[key] = override[key];
        }
    }
    return true;
}
// Suspend ends the current Clarity instance after a configured timeout period
// The way it differs from the "end" call is that it starts listening to
// user interaction events as soon as it terminates existing clarity instance.
// On the next interaction, it automatically starts another instance under a different page id
// E.g. if configured timeout is 10m, and user stays inactive for an hour.
// In this case, we will suspend clarity after 10m of inactivity and after another 50m when user interacts again
// Clarity will restart and start another instance seamlessly. Effectively not missing any active time, but also
// not holding the session during inactive time periods.
function suspend() {
    if (status) {
        event("clarity" /* Constant.Clarity */, "suspend" /* Constant.Suspend */);
        stop();
        ["mousemove", "touchstart"].forEach(x => bind(document, x, restart));
        ["resize", "scroll", "pageshow"].forEach(x => bind(window, x, restart));
    }
}
function restart() {
    restart.dn = 2 /* FunctionNames.Restart */;
    start();
    event("clarity" /* Constant.Clarity */, "restart" /* Constant.Restart */);
}

function start$4() {
    start$4.dn = 3 /* FunctionNames.DiagnosticStart */;
    start$A();
    start$f();
    start$e();
}
function stop$4() {
    stop$d();
}

var diagnostic = /*#__PURE__*/Object.freeze({
    __proto__: null,
    start: start$4,
    stop: stop$4
});

function start$3() {
    start$3.dn = 20 /* FunctionNames.LayoutStart */;
    // The order below is important 
    // and is determined by interdependencies of modules
    start$l();
    start$i();
    start$j();
    if (config$2.delayDom) {
        // Lazy load layout module as part of page load time performance improvements experiment 
        bind(window, 'load', () => {
            start$k();
        });
    }
    else {
        start$k();
    }
    start$c();
    start$n();
    start$m();
}
function stop$3() {
    stop$g();
    stop$h();
    stop$i();
    stop$j();
    stop$l();
    stop$k();
}

var layout = /*#__PURE__*/Object.freeze({
    __proto__: null,
    hashText: hashText,
    start: start$3,
    stop: stop$3
});

async function encode (type) {
    let t = time();
    let tokens = [t, type];
    switch (type) {
        case 29 /* Event.Navigation */:
            tokens.push(data.fetchStart);
            tokens.push(data.connectStart);
            tokens.push(data.connectEnd);
            tokens.push(data.requestStart);
            tokens.push(data.responseStart);
            tokens.push(data.responseEnd);
            tokens.push(data.domInteractive);
            tokens.push(data.domComplete);
            tokens.push(data.loadEventStart);
            tokens.push(data.loadEventEnd);
            tokens.push(data.redirectCount);
            tokens.push(data.size);
            tokens.push(data.type);
            tokens.push(data.protocol);
            tokens.push(data.encodedSize);
            tokens.push(data.decodedSize);
            reset();
            queue(tokens);
            break;
    }
}

let data = null;
function reset() {
    data = null;
}
function compute(entry) {
    data = {
        fetchStart: Math.round(entry.fetchStart),
        connectStart: Math.round(entry.connectStart),
        connectEnd: Math.round(entry.connectEnd),
        requestStart: Math.round(entry.requestStart),
        responseStart: Math.round(entry.responseStart),
        responseEnd: Math.round(entry.responseEnd),
        domInteractive: Math.round(entry.domInteractive),
        domComplete: Math.round(entry.domComplete),
        loadEventStart: Math.round(entry.loadEventStart),
        loadEventEnd: Math.round(entry.loadEventEnd),
        redirectCount: Math.round(entry.redirectCount),
        size: entry.transferSize ? entry.transferSize : 0,
        type: entry.type,
        protocol: entry.nextHopProtocol,
        encodedSize: entry.encodedBodySize ? entry.encodedBodySize : 0,
        decodedSize: entry.decodedBodySize ? entry.decodedBodySize : 0
    };
    encode(29 /* Event.Navigation */);
}

// Estimate variables to keep track of interactions
let interactionCountEstimate = 0;
let minKnownInteractionId = Infinity;
let maxKnownInteractionId = 0;
let prevInteractionCount = 0; // Used to track interaction count between pages
const MAX_INTERACTIONS_TO_CONSIDER = 10; // Maximum number of interactions we consider for INP
const DEFAULT_DURATION_THRESHOLD = 40; // Threshold to ignore very short interactions
// List to store the longest interaction events
const longestInteractionList = [];
// Map to track interactions by their ID, ensuring we handle duplicates
const longestInteractionMap = new Map();
/**
 * Update the approx number of interactions estimate count if the interactionCount is not supported.
 * The difference between `maxKnownInteractionId` and `minKnownInteractionId` gives us a rough range of how many interactions have occurred.
 * Dividing by 7 helps approximate the interaction count more accurately, since interaction IDs are spread out across a large range.
 */
const countInteractions = (entry) => {
    if ('interactionCount' in performance) {
        interactionCountEstimate = performance.interactionCount;
        return;
    }
    if (entry.interactionId) {
        minKnownInteractionId = Math.min(minKnownInteractionId, entry.interactionId);
        maxKnownInteractionId = Math.max(maxKnownInteractionId, entry.interactionId);
        interactionCountEstimate = maxKnownInteractionId
            ? (maxKnownInteractionId - minKnownInteractionId) / 7 + 1
            : 0;
    }
};
const getInteractionCount = () => {
    return interactionCountEstimate || 0;
};
const getInteractionCountForNavigation = () => {
    return getInteractionCount() - prevInteractionCount;
};
/**
 * Estimates the 98th percentile (P98) of the longest interactions by selecting
 * the candidate interaction based on the current interaction count.
 * Dividing by 50 is a heuristic to estimate the 98th percentile (P98) interaction.
 * This assumes one out of every 50 interactions represents the P98 interaction.
 * By dividing the total interaction count by 50, we get an index to approximate
 * the slowest 2% of interactions, helping identify a likely P98 candidate.
 */
const estimateP98LongestInteraction = () => {
    if (!longestInteractionList.length) {
        return -1;
    }
    const candidateInteractionIndex = Math.min(longestInteractionList.length - 1, Math.floor(getInteractionCountForNavigation() / 50));
    return longestInteractionList[candidateInteractionIndex].latency;
};
/**
 * Resets the interaction tracking, usually called after navigation to a new page.
 */
const resetInteractions = () => {
    prevInteractionCount = getInteractionCount();
    longestInteractionList.length = 0;
    longestInteractionMap.clear();
};
/**
 * Processes a PerformanceEventTiming entry by updating the longest interaction list.
 */
const processInteractionEntry = (entry) => {
    // Ignore entries with 0 interactionId or very short durations
    if (!entry.interactionId || entry.duration < DEFAULT_DURATION_THRESHOLD) {
        return;
    }
    countInteractions(entry);
    const minLongestInteraction = longestInteractionList[longestInteractionList.length - 1];
    const existingInteraction = longestInteractionMap.get(entry.interactionId);
    // Either update existing, add new, or replace shortest interaction if necessary
    if (existingInteraction ||
        longestInteractionList.length < MAX_INTERACTIONS_TO_CONSIDER ||
        entry.duration > (minLongestInteraction === null || minLongestInteraction === void 0 ? void 0 : minLongestInteraction.latency)) {
        if (!existingInteraction) {
            const interaction = {
                id: entry.interactionId,
                latency: entry.duration,
            };
            longestInteractionMap.set(interaction.id, interaction);
            longestInteractionList.push(interaction);
        }
        else if (entry.duration > existingInteraction.latency) {
            existingInteraction.latency = entry.duration;
        }
        longestInteractionList.sort((a, b) => b.latency - a.latency);
        // Trim the list to the maximum number of interactions to consider
        if (longestInteractionList.length > MAX_INTERACTIONS_TO_CONSIDER) {
            longestInteractionList
                .splice(MAX_INTERACTIONS_TO_CONSIDER)
                .forEach((i) => longestInteractionMap.delete(i.id));
        }
    }
};

let observer;
const types = ["navigation" /* Constant.Navigation */, "resource" /* Constant.Resource */, "longtask" /* Constant.LongTask */, "first-input" /* Constant.FID */, "layout-shift" /* Constant.CLS */, "largest-contentful-paint" /* Constant.LCP */, "event" /* Constant.PerformanceEventTiming */];
function start$2() {
    // Capture connection properties, if available
    if (navigator && "connection" in navigator) {
        log(27 /* Dimension.ConnectionType */, navigator["connection"]["effectiveType"]);
    }
    // Check the browser support performance observer as a pre-requisite for any performance measurement
    if (window["PerformanceObserver"] && PerformanceObserver.supportedEntryTypes) {
        // Start monitoring performance data after page has finished loading.
        // If the document.readyState is not yet complete, we intentionally call observe using a setTimeout.
        // This allows us to capture loadEventEnd on navigation timeline.
        if (document.readyState !== "complete") {
            bind(window, "load", setTimeout.bind(this, observe, 0));
        }
        else {
            observe();
        }
    }
    else {
        log$1(3 /* Code.PerformanceObserver */, 0 /* Severity.Info */);
    }
}
function observe() {
    observe.dn = 26 /* FunctionNames.ObserverObserve */;
    // Some browsers will throw an error for unsupported entryType, e.g. "layout-shift"
    // In those cases, we log it as a warning and continue with rest of the Clarity processing
    try {
        if (observer) {
            observer.disconnect();
        }
        observer = new PerformanceObserver(measure(handle));
        // Reference: https://developer.mozilla.org/en-US/docs/Web/API/PerformanceObserver/observe
        // "buffered" flag indicates whether buffered entries should be queued into the observer's buffer.
        // It must only be used only with the "type" option, and cannot be used with entryTypes.
        // This is why we need to individually "observe" each supported type
        for (let x of types) {
            if (PerformanceObserver.supportedEntryTypes.indexOf(x) >= 0) {
                // Initialize CLS with a value of zero. It's possible (and recommended) for sites to not have any cumulative layout shift.
                // In those cases, we want to still initialize the metric in Clarity
                if (x === "layout-shift" /* Constant.CLS */) {
                    sum(9 /* Metric.CumulativeLayoutShift */, 0);
                }
                observer.observe({ type: x, buffered: true });
            }
        }
    }
    catch (_a) {
        log$1(3 /* Code.PerformanceObserver */, 1 /* Severity.Warning */);
    }
}
function handle(entries) {
    handle.dn = 27 /* FunctionNames.ObserverHandle */;
    process(entries.getEntries());
}
function process(entries) {
    let visible = "visibilityState" in document ? document.visibilityState === "visible" : true;
    for (let i = 0; i < entries.length; i++) {
        let entry = entries[i];
        switch (entry.entryType) {
            case "navigation" /* Constant.Navigation */:
                compute(entry);
                break;
            case "resource" /* Constant.Resource */:
                let name = entry.name;
                log(4 /* Dimension.NetworkHosts */, host(name));
                if (name === config$2.upload || name === config$2.fallback) {
                    max(28 /* Metric.UploadTime */, entry.duration);
                }
                break;
            case "longtask" /* Constant.LongTask */:
                count$1(7 /* Metric.LongTaskCount */);
                break;
            case "first-input" /* Constant.FID */:
                if (visible) {
                    max(10 /* Metric.FirstInputDelay */, entry["processingStart"] - entry.startTime);
                }
                break;
            case "event" /* Constant.PerformanceEventTiming */:
                if (visible && 'PerformanceEventTiming' in window && 'interactionId' in PerformanceEventTiming.prototype) {
                    processInteractionEntry(entry);
                    // Logging it as dimension because we're always looking for the last value.
                    log(37 /* Dimension.InteractionNextPaint */, estimateP98LongestInteraction().toString());
                }
                break;
            case "layout-shift" /* Constant.CLS */:
                // Scale the value to avoid sending back floating point number
                if (visible && !entry["hadRecentInput"]) {
                    sum(9 /* Metric.CumulativeLayoutShift */, entry["value"] * 1000);
                }
                break;
            case "largest-contentful-paint" /* Constant.LCP */:
                if (visible) {
                    max(8 /* Metric.LargestPaint */, entry.startTime);
                }
                break;
        }
    }
}
function stop$2() {
    if (observer) {
        observer.disconnect();
    }
    observer = null;
    resetInteractions();
}
function host(url) {
    let a = document.createElement("a");
    a.href = url;
    return a.host;
}

function start$1() {
    start$1.dn = 25 /* FunctionNames.PerformanceStart */;
    reset();
    start$2();
}
function stop$1() {
    stop$2();
    reset();
}

var performance$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    start: start$1,
    stop: stop$1
});

const modules = [diagnostic, layout, interaction, performance$1];
function start(config$1 = null) {
    // Check that browser supports required APIs and we do not attempt to start Clarity multiple times
    if (check()) {
        config(config$1);
        start$5();
        start$B();
        modules.forEach(x => measure(x.start)());
        // If it's an internal call to start, without explicit configuration,
        // re-process any newly accumulated items in the queue
        if (config$1 === null) {
            process$8();
        }
    }
}
// By default Clarity is asynchronous and will yield by looking for requestIdleCallback.
// However, there can still be situations with single page apps where a user action can result
// in the whole DOM being destroyed and reconstructed. While Clarity will perform favorably out of the box,
// we do allow external clients to manually pause Clarity for that short burst of time and minimize
// performance impact even further. For reference, we are talking single digit milliseconds optimization here, not seconds.
function pause() {
    if (active()) {
        event("clarity" /* Constant.Clarity */, "pause" /* Constant.Pause */);
        pause$1();
    }
}
// This is how external clients can get out of pause state, and resume Clarity to continue monitoring the page
function resume() {
    if (active()) {
        resume$1();
        event("clarity" /* Constant.Clarity */, "resume" /* Constant.Resume */);
    }
}
function stop() {
    if (active()) {
        // Stop modules in the reverse order of their initialization and start queuing up items again
        modules.slice().reverse().forEach(x => measure(x.stop)());
        stop$y();
        stop$5();
        setup();
    }
}

const helper = { hash, selector, get, getNode, lookup };
const version = version$1;

exports.clarity = clarity;
exports.helper = helper;
exports.version = version;
